@@ -1,6 +1,6 @@
 #
 # jython examples for jas.
-# $Id: roots_complex_ideal.py 3759 2011-09-11 19:39:05Z kredel $
+# $Id: roots_complex_ideal.py 3892 2012-02-25 12:35:24Z kredel $
 #
 
 import sys;
@@ -14,7 +14,7 @@ from jas import startLog
 
 from jas import QQ, DD
 
-# polynomial examples: real roots over Q
+# polynomial examples: complex roots over Q
 
 r = PolyRing(QQ(),"I,x,y,z",PolyRing.lex);
 print "Ring: " + str(r);
@@ -76,13 +76,14 @@ print "G = ", G;
 print;
 
 t = System.currentTimeMillis();
-R = G.realRoots();
+R = G.complexRoots();
 t = System.currentTimeMillis() - t;
 print "R = ", R;
 print;
-print "real roots = ";
-G.realRootsPrint()
-print "real roots time =", t, "milliseconds";
+print "complex roots: ";
+G.complexRootsPrint()
+print;
+print "complex roots time =", t, "milliseconds";
 print;
 
 print "G = ", G;
