@@ -1,10 +1,9 @@
 #
 # jython examples for jas.
-# $Id: solvable.py 801 2006-03-20 21:10:02Z kredel $
+# $Id: solvable.py 4381 2013-04-27 09:57:28Z kredel $
 #
 
 from jas import SolvableRing
-from jas import SolvableIdeal
 
 
 # WA_32 solvable polynomial example
@@ -32,7 +31,7 @@ ps = """
 )
 """;
 
-f = SolvableIdeal( r, ps );
+f = r.ideal( ps );
 print "SolvableIdeal: " + str(f);
 print;
 
