@@ -1,10 +1,9 @@
 #
 # jython examples for jas.
-# $Id: syzsolv.py 2418 2009-02-07 14:20:47Z kredel $
+# $Id: syzsolv.py 4381 2013-04-27 09:57:28Z kredel $
 #
 
 from jas import SolvableRing
-from jas import SolvableIdeal
 
 # ? example
 
@@ -29,7 +28,7 @@ ps = """
 )
 """;
 
-f = SolvableIdeal( r, ps );
+f = r.ideal( ps );
 print "SolvIdeal: " + str(f);
 print;
 
