@@ -1,5 +1,5 @@
 /*
- * $Id: BigRational.java 3652 2011-06-02 18:17:04Z kredel $
+ * $Id: BigRational.java 4125 2012-08-19 19:05:22Z kredel $
  */
 
 package edu.jas.arith;
@@ -15,8 +15,8 @@ import java.util.List;
 import java.util.Random;
 import java.util.Set;
 
-import edu.jas.kern.StringUtil;
 import edu.jas.kern.Scripting;
+import edu.jas.kern.StringUtil;
 import edu.jas.structure.GcdRingElem;
 import edu.jas.structure.Power;
 import edu.jas.structure.RingFactory;
@@ -24,32 +24,25 @@ import edu.jas.structure.RingFactory;
 
 /**
  * Immutable arbitrary-precision rational numbers. BigRational class based on
- * BigInteger implementing the RingElem interface and with the familiar SAC
- * static method names. BigInteger is from java.math in the implementation.
+ * BigInteger and implementing the RingElem interface. BigInteger is from
+ * java.math in the implementation. The SAC2 static methods are also provided.
  * @author Heinz Kredel
  */
 
 public final class BigRational implements GcdRingElem<BigRational>, RingFactory<BigRational>, Rational,
-        Iterable<BigRational> {
+                Iterable<BigRational> {
 
 
     /**
      * Numerator part of the data structure.
      */
-    protected final BigInteger num;
+    public final BigInteger num;
 
 
     /**
      * Denominator part of the data structure.
      */
-    protected final BigInteger den;
-
-
-    /* from history: */
-    private final static BigInteger IZERO = BigInteger.ZERO;
-
-
-    private final static BigInteger IONE = BigInteger.ONE;
+    public final BigInteger den;
 
 
     /**
@@ -85,7 +78,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
      */
     public BigRational(BigInteger n) {
         num = n;
-        den = IONE; // be aware of static initialization order
+        den = BigInteger.ONE; // be aware of static initialization order
         //den = BigInteger.ONE;
     }
 
@@ -99,6 +92,20 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
     }
 
 
+    /**
+     * Constructor for a BigRational from jas.arith.BigIntegers.
+     * @param n edu.jas.arith.BigInteger.
+     * @param d edu.jas.arith.BigInteger.
+     */
+    public BigRational(edu.jas.arith.BigInteger n, edu.jas.arith.BigInteger d) {
+        BigInteger nu = n.getVal();
+        BigInteger de = d.getVal();
+        BigRational r = RNRED(nu, de);
+        num = r.num;
+        den = r.den;
+    }
+
+
     /**
      * Constructor for a BigRational from longs.
      * @param n long.
@@ -119,7 +126,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
      */
     public BigRational(long n) {
         num = BigInteger.valueOf(n);
-        den = IONE;
+        den = BigInteger.ONE;
     }
 
 
@@ -127,8 +134,8 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
      * Constructor for a BigRational with no arguments.
      */
     public BigRational() {
-        num = IZERO;
-        den = IONE;
+        num = BigInteger.ZERO;
+        den = BigInteger.ONE;
     }
 
 
@@ -139,13 +146,13 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
      */
     public BigRational(String s) throws NumberFormatException {
         if (s == null) {
-            num = IZERO;
-            den = IONE;
+            num = BigInteger.ZERO;
+            den = BigInteger.ONE;
             return;
         }
         if (s.length() == 0) {
-            num = IZERO;
-            den = IONE;
+            num = BigInteger.ZERO;
+            den = BigInteger.ONE;
             return;
         }
         BigInteger n;
@@ -229,7 +236,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
      * @see java.lang.Object#clone()
      */
     @Override
-    public BigRational clone() {
+    public BigRational copy() {
         return new BigRational(num, den);
     }
 
@@ -313,7 +320,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
             s.append(num.toString());
             return s.toString();
         }
-        switch (Scripting.getLang() ) {
+        switch (Scripting.getLang()) {
         case Python:
             s.append("(");
             s.append(num.toString());
@@ -393,7 +400,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
      * @return characteristic of this ring.
      */
     public java.math.BigInteger characteristic() {
-        return java.math.BigInteger.ZERO;
+        return BigInteger.ZERO;
     }
 
 
@@ -407,6 +414,16 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
     }
 
 
+    /**
+     * Get a BigRational element from a arith.BigInteger.
+     * @param a arith.BigInteger.
+     * @return BigRational from a.
+     */
+    public BigRational fromInteger(edu.jas.arith.BigInteger a) {
+        return new BigRational(a);
+    }
+
+
     /**
      * Get a BigRational element from a math.BigInteger.
      * @param a math.BigInteger.
@@ -500,9 +517,9 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
     public static BigRational RNRED(BigInteger n, BigInteger d) {
         BigInteger num;
         BigInteger den;
-        if (n.equals(IZERO)) {
+        if (n.equals(BigInteger.ZERO)) {
             num = n;
-            den = IONE;
+            den = BigInteger.ONE;
             return new BigRational(num, den);
         }
         BigInteger C = n.gcd(d);
@@ -767,12 +784,12 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
         R2 = den; //this.denominator();
         S1 = S.num;
         S2 = S.den;
-        if (R2.equals(IONE) && S2.equals(IONE)) {
+        if (R2.equals(BigInteger.ONE) && S2.equals(BigInteger.ONE)) {
             T1 = R1.multiply(S1);
-            T = new BigRational(T1, IONE);
+            T = new BigRational(T1, BigInteger.ONE);
             return T;
         }
-        if (R2.equals(IONE)) {
+        if (R2.equals(BigInteger.ONE)) {
             D1 = R1.gcd(S2);
             RB1 = R1.divide(D1);
             SB2 = S2.divide(D1);
@@ -780,7 +797,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
             T = new BigRational(T1, SB2);
             return T;
         }
-        if (S2.equals(IONE)) {
+        if (S2.equals(BigInteger.ONE)) {
             D2 = S1.gcd(R2);
             SB1 = S1.divide(D2);
             RB2 = R2.divide(D2);
@@ -880,7 +897,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
             A = A.negate();
         }
         B = new BigInteger(n, rnd); // always positive
-        B = B.add(IONE);
+        B = B.add(BigInteger.ONE);
         return RNRED(A, B);
     }
 
@@ -950,18 +967,18 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
         R2 = den; //this.denominator();
         S1 = S.num;
         S2 = S.den;
-        if (R2.equals(IONE) && S2.equals(IONE)) {
+        if (R2.equals(BigInteger.ONE) && S2.equals(BigInteger.ONE)) {
             T1 = R1.add(S1);
-            T = new BigRational(T1, IONE);
+            T = new BigRational(T1, BigInteger.ONE);
             return T;
         }
-        if (R2.equals(IONE)) {
+        if (R2.equals(BigInteger.ONE)) {
             T1 = R1.multiply(S2);
             T1 = T1.add(S1);
             T = new BigRational(T1, S2);
             return T;
         }
-        if (S2.equals(IONE)) {
+        if (S2.equals(BigInteger.ONE)) {
             T1 = R2.multiply(S1);
             T1 = T1.add(R1);
             T = new BigRational(T1, R2);
@@ -973,13 +990,13 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
         J1Y = R1.multiply(SB2);
         J2Y = RB2.multiply(S1);
         T1 = J1Y.add(J2Y);
-        if (T1.equals(IZERO)) {
+        if (T1.equals(BigInteger.ZERO)) {
             T = ZERO;
             return T;
         }
-        if (!D.equals(IONE)) {
+        if (!D.equals(BigInteger.ONE)) {
             E = T1.gcd(D);
-            if (!E.equals(IONE)) {
+            if (!E.equals(BigInteger.ONE)) {
                 T1 = T1.divide(E);
                 R2 = R2.divide(E);
             }
@@ -1105,7 +1122,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
 
 
     /**
-     * Get a BigInteger iterator.
+     * Get a BigRational iterator.
      * @return a iterator over all rationals.
      */
     public Iterator<BigRational> iterator() {
@@ -1117,7 +1134,7 @@ public final class BigRational implements GcdRingElem<BigRational>, RingFactory<
 
 
     /**
-     * Get a BigInteger iterator with no duplicates.
+     * Get a BigRational iterator with no duplicates.
      * @return a iterator over all rationals without duplicates.
      */
     public Iterator<BigRational> uniqueIterator() {
@@ -1188,8 +1205,8 @@ class BigRationalIterator implements Iterator<BigRational> {
         curr = edu.jas.arith.BigRational.ZERO;
         level = 0;
         den = new edu.jas.arith.BigInteger(); // ZERO
-        num = edu.jas.arith.BigInteger.ONE.clone();
-        if (nn) {
+        num = edu.jas.arith.BigInteger.ONE.copy();
+        if (nonNegative) {
             den.setNonNegativeIterator();
         } else {
             den.setAllIterator();
@@ -1202,6 +1219,9 @@ class BigRationalIterator implements Iterator<BigRational> {
         @SuppressWarnings("unused")
         edu.jas.arith.BigInteger unused = denit.next(); // skip zero denominator
         unused = numit.next();
+        if (unused == null) { // use for findbugs
+            System.out.println("unused is null");
+        }
         denlist.add(denit.next());
         numlist.add(numit.next());
         denlistit = denlist.iterator();
@@ -1266,7 +1286,7 @@ class BigRationalIterator implements Iterator<BigRational> {
 
 
 /**
- * Big rational unique iterator. Uses Cantors diagonal enumeration, produces all
+ * Big rational unique iterator. Uses Cantors diagonal enumeration, produces
  * distinct elements.
  * @author Heinz Kredel
  */
