@@ -1,5 +1,5 @@
 /*
- * $Id: ElementaryIntegration.java 3356 2010-10-23 16:41:01Z kredel $
+ * $Id: ElementaryIntegration.java 4125 2012-08-19 19:05:22Z kredel $
  */
 
 package edu.jas.integrate;
@@ -361,7 +361,7 @@ public class ElementaryIntegration<C extends GcdRingElem<C>> {
         // = GCDFactory.<C>getImplementation( cfac.coFac );
         GreatestCommonDivisorAbstract<AlgebraicNumber<C>> aengine = null;
 
-        GenPolynomial<GenPolynomial<C>> Rc = engine.recursiveResultant(Pc, At);
+        GenPolynomial<GenPolynomial<C>> Rc = engine.recursiveUnivariateResultant(Pc, At);
         //System.out.println("Rc = " + Rc);
         GenPolynomial<C> res = Rc.leadingBaseCoefficient();
         //no: res = res.monic();
@@ -377,8 +377,8 @@ public class ElementaryIntegration<C extends GcdRingElem<C>> {
             }
             //vars = new String[] { "z_" + Math.abs(r.hashCode() % 1000) };
             vars = pfac.newVars("z_");
-            pfac = pfac.clone();
-            vars = pfac.setVars(vars);
+            pfac = pfac.copy();
+            String[] unused = pfac.setVars(vars);
             r = pfac.copy(r); // hack to exchange the variables
             //System.out.println("r(z_) = " + r);
             AlgebraicNumberRing<C> afac = new AlgebraicNumberRing<C>(r, true); // since irreducible
