@@ -1,5 +1,5 @@
 /*
- * $Id: AlgebraicNumberRing.java 3487 2011-01-10 22:39:18Z kredel $
+ * $Id: AlgebraicNumberRing.java 4057 2012-07-26 20:35:44Z kredel $
  */
 
 package edu.jas.poly;
@@ -14,7 +14,7 @@ import java.util.Random;
 import org.apache.log4j.Logger;
 
 import edu.jas.kern.Scripting;
-import edu.jas.structure.GcdRingElem;
+import edu.jas.structure.RingElem;
 import edu.jas.structure.RingFactory;
 import edu.jas.util.CartesianProduct;
 import edu.jas.util.CartesianProductInfinite;
@@ -26,8 +26,8 @@ import edu.jas.util.CartesianProductInfinite;
  * @author Heinz Kredel
  */
 
-public class AlgebraicNumberRing<C extends GcdRingElem<C>> implements RingFactory<AlgebraicNumber<C>>,
-        Iterable<AlgebraicNumber<C>> {
+public class AlgebraicNumberRing<C extends RingElem<C>> implements RingFactory<AlgebraicNumber<C>>,
+                Iterable<AlgebraicNumber<C>> {
 
 
     /**
@@ -303,7 +303,7 @@ public class AlgebraicNumberRing<C extends GcdRingElem<C>> implements RingFactor
     @Override
     public String toString() {
         return "AlgebraicNumberRing[ " + modul.toString() + " | isField=" + isField + " :: "
-                + ring.toString() + " ]";
+                        + ring.toString() + " ]";
     }
 
 
@@ -317,7 +317,7 @@ public class AlgebraicNumberRing<C extends GcdRingElem<C>> implements RingFactor
         StringBuffer s = new StringBuffer();
         s.append("AN(");
         s.append(modul.toScript());
-        switch (Scripting.getLang() ) {
+        switch (Scripting.getLang()) {
         case Ruby:
             s.append((isField() ? ",true" : ",false"));
             break;
@@ -419,7 +419,7 @@ public class AlgebraicNumberRing<C extends GcdRingElem<C>> implements RingFactor
      * @return S, with S mod c.modul == c and S mod a.modul == a.
      */
     public AlgebraicNumber<C> chineseRemainder(AlgebraicNumber<C> c, AlgebraicNumber<C> ci,
-            AlgebraicNumber<C> a) {
+                    AlgebraicNumber<C> a) {
         if (true) { // debug
             if (c.ring.modul.compareTo(a.ring.modul) < 1) {
                 System.out.println("AlgebraicNumber error " + c + ", " + a);
@@ -532,7 +532,7 @@ public class AlgebraicNumberRing<C extends GcdRingElem<C>> implements RingFactor
  * Algebraic number iterator.
  * @author Heinz Kredel
  */
-class AlgebraicNumberIterator<C extends GcdRingElem<C>> implements Iterator<AlgebraicNumber<C>> {
+class AlgebraicNumberIterator<C extends RingElem<C>> implements Iterator<AlgebraicNumber<C>> {
 
 
     /**
@@ -555,7 +555,7 @@ class AlgebraicNumberIterator<C extends GcdRingElem<C>> implements Iterator<Alge
 
     /**
      * CartesianProduct iterator constructor.
-     * @param comps components of the cartesian product.
+     * @param aring AlgebraicNumberRing components of the Cartesian product.
      */
     public AlgebraicNumberIterator(AlgebraicNumberRing<C> aring) {
         RingFactory<C> cf = aring.ring.coFac;
@@ -582,6 +582,9 @@ class AlgebraicNumberIterator<C extends GcdRingElem<C>> implements Iterator<Alge
             CartesianProductInfinite<C> tuples = new CartesianProductInfinite<C>(comps);
             iter = tuples.iterator();
         }
+        if (logger.isInfoEnabled()) {
+            logger.info("iterator for degree " + d + ", finite = " + cf.isFinite());
+        }
     }
 
 
