@@ -1,11 +1,12 @@
 /*
- * $Id: ExpVectorByte.java 3295 2010-08-26 17:01:10Z kredel $
+ * $Id: ExpVectorByte.java 4270 2012-10-24 20:41:11Z kredel $
  */
 
 package edu.jas.poly;
 
 
-import java.util.Vector;
+import java.util.List;
+import java.util.ArrayList;
 
 
 /**
@@ -16,7 +17,7 @@ import java.util.Vector;
  * @author Heinz Kredel
  */
 
-public class ExpVectorByte extends ExpVector
+public final class ExpVectorByte extends ExpVector
 /*implements AbelianGroupElem<ExpVectorByte>*/{
 
 
@@ -29,13 +30,13 @@ public class ExpVectorByte extends ExpVector
     /**
      * Largest byte.
      */
-    public static long maxByte = (long) Byte.MAX_VALUE / 2;
+    public static final long maxByte = (long) Byte.MAX_VALUE / 2;
 
 
     /**
      * Smallest byte.
      */
-    public static long minByte = (long) Byte.MIN_VALUE / 2;
+    public static final long minByte = (long) Byte.MIN_VALUE / 2;
 
 
     /**
@@ -107,7 +108,7 @@ public class ExpVectorByte extends ExpVector
     public ExpVectorByte(String s) throws NumberFormatException {
         super();
         // first format = (1,2,3,4,5,6,7)
-        Vector<Byte> exps = new Vector<Byte>();
+        List<Byte> exps = new ArrayList<Byte>();
         s = s.trim();
         int b = s.indexOf('(');
         int e = s.indexOf(')', b + 1);
@@ -119,18 +120,18 @@ public class ExpVectorByte extends ExpVector
             while ((k = s.indexOf(',', b)) >= 0) {
                 teil = s.substring(b, k);
                 a = Byte.parseByte(teil);
-                exps.add(new Byte(a));
+                exps.add(Byte.valueOf(a));
                 b = k + 1;
             }
             if (b <= e) {
                 teil = s.substring(b, e);
                 a = Byte.parseByte(teil);
-                exps.add(new Byte(a));
+                exps.add(Byte.valueOf(a));
             }
             int length = exps.size();
             val = new byte[length];
             for (int j = 0; j < length; j++) {
-                val[j] = exps.elementAt(j).byteValue();
+                val[j] = exps.get(j).byteValue();
             }
         } else {
             // not implemented
@@ -147,7 +148,7 @@ public class ExpVectorByte extends ExpVector
      * @see java.lang.Object#clone()
      */
     @Override
-    public ExpVectorByte clone() {
+    public ExpVectorByte copy() {
         byte[] w = new byte[val.length];
         System.arraycopy(val, 0, w, 0, val.length);
         return new ExpVectorByte(w);
@@ -366,6 +367,16 @@ public class ExpVectorByte extends ExpVector
     }
 
 
+    /**
+     * hashCode for this exponent vector.
+     * @see java.lang.Object#hashCode() Only for findbugs.
+     */
+    @Override
+    public int hashCode() {
+        return super.hashCode();
+    }
+
+
     /**
      * ExpVector absolute value.
      * @return abs(this).
@@ -445,7 +456,8 @@ public class ExpVectorByte extends ExpVector
      * @return substituted ExpVector.
      */
     public ExpVectorByte subst(int i, byte d) {
-        ExpVectorByte V = (ExpVectorByte) this.clone();
+        ExpVectorByte V = this.copy();
+        @SuppressWarnings("unused")
         long e = V.setVal(i, d);
         return V;
         //return EVSU(this, i, d);
@@ -460,7 +472,8 @@ public class ExpVectorByte extends ExpVector
      */
     @Override
     public ExpVectorByte subst(int i, long d) {
-        ExpVectorByte V = (ExpVectorByte) this.clone();
+        ExpVectorByte V = this.copy();
+        @SuppressWarnings("unused")
         long e = V.setVal(i, d);
         return V;
         //return EVSU(this, i, d);
@@ -636,7 +649,6 @@ public class ExpVectorByte extends ExpVector
      * @param V
      * @return 0 if U == V, -1 if U &lt; V, 1 if U &gt; V.
      */
-    //@Override
     @Override
     public int compareTo(ExpVector V) {
         return this.invLexCompareTo(V);
