@@ -1,5 +1,5 @@
 /*
- * $Id: GenPolynomial.java 3800 2011-10-12 21:36:11Z kredel $
+ * $Id: GenPolynomial.java 4391 2013-04-28 15:46:54Z kredel $
  */
 
 package edu.jas.poly;
@@ -140,12 +140,10 @@ Iterable<Monomial<C>> {
 
 
     /**
-     * Clone this GenPolynomial.
-     * @see java.lang.Object#clone()
+     * Copy this GenPolynomial.
+     * @return copy of this.
      */
-    @Override
-    public GenPolynomial<C> clone() {
-        //return ring.copy(this);
+    public GenPolynomial<C> copy() {
         return new GenPolynomial<C>(ring, this.val);
     }
 
@@ -178,10 +176,10 @@ Iterable<Monomial<C>> {
      * @param e exponent.
      */
     public void doPutToMap(ExpVector e, C c) {
-        if (false && debug) {
+        if (debug) {
             C a = val.get(e);
             if (a != null) {
-                logger.info("map entry exists " + e + " to " + a + " new " + c);
+                logger.error("map entry exists " + e + " to " + a + " new " + c);
             }
         }
         if (!c.isZERO()) {
@@ -190,6 +188,27 @@ Iterable<Monomial<C>> {
     }
 
 
+    /**
+     * Remove an ExpVector to coefficient entry from the internal map of this
+     * GenPolynomial. <b>Note:</b> Do not use this method unless you are
+     * constructing a new polynomial. this is modified and breaks the
+     * immutability promise of this class.
+     * @param e exponent.
+     * @param c expected coefficient, null for ignore.
+     */
+    public void doRemoveFromMap(ExpVector e, C c) {
+        C b = val.remove(e);
+        if (debug) {
+            if ( c == null ) { // ignore b
+                return;
+            }
+            if ( ! c.equals(b) ) {
+                logger.error("map entry wrong " + e + " to " + c + " old " + b);
+            }
+        }
+    }
+
+
     /**
      * Put an a sorted map of exponents to coefficients into the internal map of
      * this GenPolynomial. <b>Note:</b> Do not use this method unless you are
@@ -200,10 +219,10 @@ Iterable<Monomial<C>> {
     public void doPutToMap(SortedMap<ExpVector, C> vals) {
         for (Map.Entry<ExpVector, C> me : vals.entrySet()) {
             ExpVector e = me.getKey();
-            if (false && debug) {
+            if (debug) {
                 C a = val.get(e);
                 if (a != null) {
-                    logger.info("map entry exists " + e + " to " + a + " new " + me.getValue());
+                    logger.error("map entry exists " + e + " to " + a + " new " + me.getValue());
                 }
             }
             C c = me.getValue();
@@ -273,12 +292,13 @@ Iterable<Monomial<C>> {
                     }
                     ExpVector e = m.getKey();
                     if (!c.isONE() || e.isZERO()) {
+                        String cs = c.toString();
                         if (c instanceof GenPolynomial || c instanceof AlgebraicNumber) {
-                            s.append("{ ");
-                        }
-                        s.append(c.toString());
-                        if (c instanceof GenPolynomial || c instanceof AlgebraicNumber) {
-                            s.append(" }");
+                            s.append("( ");
+                            s.append(cs);
+                            s.append(" )");
+                        } else {
+                            s.append(cs);
                         }
                         s.append(" ");
                     }
@@ -342,11 +362,11 @@ Iterable<Monomial<C>> {
             v = GenPolynomialRing.newVars("x", ring.nvar);
         }
         boolean parenthesis = false;
-        if (ring.coFac instanceof GenPolynomialRing || ring.coFac instanceof AlgebraicNumberRing
-        // || ring.coFac instanceof RealAlgebraicRing
-        ) {
+        //if (ring.coFac instanceof GenPolynomialRing || ring.coFac instanceof AlgebraicNumberRing
+        //    || ring.coFac instanceof RealAlgebraicRing
+        //) {
             // inactive: parenthesis = true;
-        }
+        //}
         boolean first = true;
         for (Map.Entry<ExpVector, C> m : val.entrySet()) {
             C c = m.getValue();
@@ -417,10 +437,7 @@ Iterable<Monomial<C>> {
         if (c == null) {
             return false;
         }
-        if (!c.isONE()) {
-            return false;
-        }
-        return true;
+        return c.isONE();
     }
 
 
@@ -437,10 +454,7 @@ Iterable<Monomial<C>> {
         if (c == null) {
             return false;
         }
-        if (c.isUnit()) {
-            return true;
-        }
-        return false;
+        return c.isUnit();
     }
 
 
@@ -461,6 +475,26 @@ Iterable<Monomial<C>> {
     }
 
 
+    /**
+     * Is GenPolynomial&lt;C&gt; homogeneous.
+     * @return true, if this is homogeneous, else false.
+     */
+    public boolean isHomogeneous() {
+        if (val.size() <= 1) {
+            return true;
+        }
+        long deg = -1;
+        for (ExpVector e : val.keySet()) {
+            if (deg < 0) {
+                deg = e.totalDeg();
+            } else if (deg != e.totalDeg()) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+
     /**
      * Comparison with any other object.
      * @see java.lang.Object#equals(java.lang.Object)
@@ -507,21 +541,23 @@ Iterable<Monomial<C>> {
         }
         SortedMap<ExpVector, C> av = this.val;
         SortedMap<ExpVector, C> bv = b.val;
-        Iterator<ExpVector> ai = av.keySet().iterator();
-        Iterator<ExpVector> bi = bv.keySet().iterator();
+        Iterator<Map.Entry<ExpVector, C>> ai = av.entrySet().iterator();
+        Iterator<Map.Entry<ExpVector, C>> bi = bv.entrySet().iterator();
         int s = 0;
         int c = 0;
         while (ai.hasNext() && bi.hasNext()) {
-            ExpVector ae = ai.next();
-            ExpVector be = bi.next();
+            Map.Entry<ExpVector, C> aie = ai.next();
+            Map.Entry<ExpVector, C> bie = bi.next();
+            ExpVector ae = aie.getKey();
+            ExpVector be = bie.getKey();
             s = ae.compareTo(be);
             //System.out.println("s = " + s + ", " + ae + ", " +be);
             if (s != 0) {
                 return s;
             }
             if (c == 0) {
-                C ac = av.get(ae);
-                C bc = bv.get(be);
+                C ac = aie.getValue(); //av.get(ae);
+                C bc = bie.getValue(); //bv.get(be);
                 c = ac.compareTo(bc);
             }
         }
@@ -577,7 +613,7 @@ Iterable<Monomial<C>> {
      */
     public ExpVector leadingExpVector() {
         if (val.size() == 0) {
-            return null; // ring.evzero or null ?;
+            return null; // ring.evzero? needs many changes 
         }
         return val.firstKey();
     }
@@ -589,7 +625,7 @@ Iterable<Monomial<C>> {
      */
     public ExpVector trailingExpVector() {
         if (val.size() == 0) {
-            return null; // ring.evzero or null ?;
+            return ring.evzero; // or null ?;
         }
         return val.lastKey();
     }
@@ -658,7 +694,12 @@ Iterable<Monomial<C>> {
         if (val.size() == 0) {
             return 0; // 0 or -1 ?;
         }
-        int j = ring.nvar - 1 - i;
+        int j;
+        if (i >= 0) {
+            j = ring.nvar - 1 - i;
+        } else { // python like -1 means main variable
+            j = ring.nvar + i;
+        }
         long deg = 0;
         for (ExpVector e : val.keySet()) {
             long d = e.getVal(j);
@@ -689,6 +730,25 @@ Iterable<Monomial<C>> {
     }
 
 
+    /**
+     * Total degree.
+     * @return total degree in any variables.
+     */
+    public long totalDegree() {
+        if (val.size() == 0) {
+            return 0; // 0 or -1 ?;
+        }
+        long deg = 0;
+        for (ExpVector e : val.keySet()) {
+            long d = e.totalDeg();
+            if (d > deg) {
+                deg = d;
+            }
+        }
+        return deg;
+    }
+
+
     /**
      * Maximal degree vector.
      * @return maximal degree vector of all variables.
@@ -752,12 +812,13 @@ Iterable<Monomial<C>> {
             return S;
         }
         assert (ring.nvar == S.ring.nvar);
-        GenPolynomial<C> n = this.clone(); //new GenPolynomial<C>(ring, val); 
+        GenPolynomial<C> n = this.copy(); //new GenPolynomial<C>(ring, val); 
         SortedMap<ExpVector, C> nv = n.val;
         SortedMap<ExpVector, C> sv = S.val;
-        for (ExpVector e : sv.keySet()) {
+        for (Map.Entry<ExpVector, C> me : sv.entrySet()) {
+            ExpVector e = me.getKey();
+            C y = me.getValue(); //sv.get(e); // assert y != null
             C x = nv.get(e);
-            C y = sv.get(e); // assert y != null
             if (x != null) {
                 x = x.sum(y);
                 if (!x.isZERO()) {
@@ -787,7 +848,7 @@ Iterable<Monomial<C>> {
         if (a.isZERO()) {
             return this;
         }
-        GenPolynomial<C> n = this.clone(); //new GenPolynomial<C>(ring, val); 
+        GenPolynomial<C> n = this.copy(); //new GenPolynomial<C>(ring, val); 
         SortedMap<ExpVector, C> nv = n.val;
         //if ( nv.size() == 0 ) { nv.put(e,a); return n; }
         C x = nv.get(e);
@@ -832,12 +893,13 @@ Iterable<Monomial<C>> {
             return S.negate();
         }
         assert (ring.nvar == S.ring.nvar);
-        GenPolynomial<C> n = this.clone(); //new GenPolynomial<C>(ring, val); 
+        GenPolynomial<C> n = this.copy(); //new GenPolynomial<C>(ring, val); 
         SortedMap<ExpVector, C> nv = n.val;
         SortedMap<ExpVector, C> sv = S.val;
-        for (ExpVector e : sv.keySet()) {
+        for (Map.Entry<ExpVector, C> me : sv.entrySet()) {
+            ExpVector e = me.getKey();
+            C y = me.getValue(); //sv.get(e); // assert y != null
             C x = nv.get(e);
-            C y = sv.get(e); // assert y != null
             if (x != null) {
                 x = x.subtract(y);
                 if (!x.isZERO()) {
@@ -867,7 +929,7 @@ Iterable<Monomial<C>> {
         if (a.isZERO()) {
             return this;
         }
-        GenPolynomial<C> n = this.clone(); //new GenPolynomial<C>(ring, val); 
+        GenPolynomial<C> n = this.copy(); 
         SortedMap<ExpVector, C> nv = n.val;
         C x = nv.get(e);
         if (x != null) {
@@ -895,12 +957,199 @@ Iterable<Monomial<C>> {
     }
 
 
+    /**
+     * GenPolynomial subtract a multiple.
+     * @param a coefficient.
+     * @param S GenPolynomial.
+     * @return this - a x<sup>e</sup> S.
+     */
+    public GenPolynomial<C> subtractMultiple(C a, GenPolynomial<C> S) {
+        if (a == null) {
+            return this;
+        }
+        if (a.isZERO()) {
+            return this;
+        }
+        if (S == null) {
+            return this;
+        }
+        if (S.isZERO()) {
+            return this;
+        }
+        if (this.isZERO()) {
+            return S.multiply(a.negate());
+        }
+        assert (ring.nvar == S.ring.nvar);
+        GenPolynomial<C> n = this.copy(); 
+        SortedMap<ExpVector, C> nv = n.val;
+        SortedMap<ExpVector, C> sv = S.val;
+        for (Map.Entry<ExpVector, C> me : sv.entrySet()) {
+            ExpVector f = me.getKey();
+            C y = me.getValue(); // assert y != null
+            y = a.multiply(y);
+            C x = nv.get(f);
+            if (x != null) {
+                x = x.subtract(y);
+                if (!x.isZERO()) {
+                    nv.put(f, x);
+                } else {
+                    nv.remove(f);
+                }
+            } else if ( !y.isZERO() ) {
+                nv.put(f, y.negate());
+            }
+        }
+        return n;
+    }
+
+
+    /**
+     * GenPolynomial subtract a multiple.
+     * @param a coefficient.
+     * @param e exponent.
+     * @param S GenPolynomial.
+     * @return this - a x<sup>e</sup> S.
+     */
+    public GenPolynomial<C> subtractMultiple(C a, ExpVector e, GenPolynomial<C> S) {
+        if (a == null) {
+            return this;
+        }
+        if (a.isZERO()) {
+            return this;
+        }
+        if (S == null) {
+            return this;
+        }
+        if (S.isZERO()) {
+            return this;
+        }
+        if (this.isZERO()) {
+            return S.multiply(a.negate(),e);
+        }
+        assert (ring.nvar == S.ring.nvar);
+        GenPolynomial<C> n = this.copy(); 
+        SortedMap<ExpVector, C> nv = n.val;
+        SortedMap<ExpVector, C> sv = S.val;
+        for (Map.Entry<ExpVector, C> me : sv.entrySet()) {
+            ExpVector f = me.getKey();
+            f = e.sum(f);
+            C y = me.getValue(); // assert y != null
+            y = a.multiply(y);
+            C x = nv.get(f);
+            if (x != null) {
+                x = x.subtract(y);
+                if (!x.isZERO()) {
+                    nv.put(f, x);
+                } else {
+                    nv.remove(f);
+                }
+            } else if ( !y.isZERO() ) {
+                nv.put(f, y.negate());
+            }
+        }
+        return n;
+    }
+
+
+    /**
+     * GenPolynomial scale and subtract a multiple.
+     * @param b scale factor.
+     * @param a coefficient.
+     * @param e exponent.
+     * @param S GenPolynomial.
+     * @return this * b - a x<sup>e</sup> S.
+     */
+    public GenPolynomial<C> scaleSubtractMultiple(C b, C a, ExpVector e, GenPolynomial<C> S) {
+        if (a == null || S == null) {
+            return this.multiply(b);
+        }
+        if (a.isZERO() || S.isZERO()) {
+            return this.multiply(b);
+        }
+        if (this.isZERO() || b == null || b.isZERO()) {
+            return S.multiply(a.negate(),e);
+        }
+        if (b.isONE()) {
+            return subtractMultiple(a,e,S);
+        }
+        assert (ring.nvar == S.ring.nvar);
+        GenPolynomial<C> n = this.multiply(b);
+        SortedMap<ExpVector, C> nv = n.val;
+        SortedMap<ExpVector, C> sv = S.val;
+        for (Map.Entry<ExpVector, C> me : sv.entrySet()) {
+            ExpVector f = me.getKey();
+            f = e.sum(f);
+            C y = me.getValue(); // assert y != null
+            y = a.multiply(y); // now y can be zero
+            C x = nv.get(f);
+            if (x != null) {
+                x = x.subtract(y);
+                if (!x.isZERO()) {
+                    nv.put(f, x);
+                } else {
+                    nv.remove(f);
+                }
+            } else if ( !y.isZERO() ) {
+                nv.put(f, y.negate());
+            }
+        }
+        return n;
+    }
+
+
+    /**
+     * GenPolynomial scale and subtract a multiple.
+     * @param b scale factor.
+     * @param g scale exponent.
+     * @param a coefficient.
+     * @param e exponent.
+     * @param S GenPolynomial.
+     * @return this * a x<sup>g</sup> - a x<sup>e</sup> S.
+     */
+    public GenPolynomial<C> scaleSubtractMultiple(C b, ExpVector g, C a, ExpVector e, GenPolynomial<C> S) {
+        if (a == null || S == null) {
+            return this.multiply(b,g);
+        }
+        if (a.isZERO() || S.isZERO()) {
+            return this.multiply(b,g);
+        }
+        if (this.isZERO() || b == null || b.isZERO()) {
+            return S.multiply(a.negate(),e);
+        }
+        if (b.isONE() && g.isZERO()) {
+            return subtractMultiple(a,e,S);
+        }
+        assert (ring.nvar == S.ring.nvar);
+        GenPolynomial<C> n = this.multiply(b,g);
+        SortedMap<ExpVector, C> nv = n.val;
+        SortedMap<ExpVector, C> sv = S.val;
+        for (Map.Entry<ExpVector, C> me : sv.entrySet()) {
+            ExpVector f = me.getKey();
+            f = e.sum(f);
+            C y = me.getValue(); // assert y != null
+            y = a.multiply(y); // y can be zero now
+            C x = nv.get(f);
+            if (x != null) {
+                x = x.subtract(y);
+                if (!x.isZERO()) {
+                    nv.put(f, x);
+                } else {
+                    nv.remove(f);
+                }
+            } else if ( !y.isZERO() ) {
+                nv.put(f, y.negate());
+            }
+        }
+        return n;
+    }
+
+
     /**
      * GenPolynomial negation.
      * @return -this.
      */
     public GenPolynomial<C> negate() {
-        GenPolynomial<C> n = ring.getZERO().clone();
+        GenPolynomial<C> n = ring.getZERO().copy();
         //new GenPolynomial<C>(ring, ring.getZERO().val);
         SortedMap<ExpVector, C> v = n.val;
         for (Map.Entry<ExpVector, C> m : val.entrySet()) {
@@ -942,12 +1191,12 @@ Iterable<Monomial<C>> {
         assert (ring.nvar == S.ring.nvar);
         if (this instanceof GenSolvablePolynomial || S instanceof GenSolvablePolynomial) {
             //throw new RuntimeException("wrong method dispatch in JRE ");
-            logger.warn("wrong method dispatch in JRE (S) - trying to fix");
+            logger.debug("warn: wrong method dispatch in JRE multiply(S) - trying to fix");
             GenSolvablePolynomial<C> T = (GenSolvablePolynomial<C>) this;
             GenSolvablePolynomial<C> Sp = (GenSolvablePolynomial<C>) S;
             return T.multiply(Sp);
         }
-        GenPolynomial<C> p = ring.getZERO().clone();
+        GenPolynomial<C> p = ring.getZERO().copy();
         SortedMap<ExpVector, C> pv = p.val;
         for (Map.Entry<ExpVector, C> m1 : val.entrySet()) {
             C c1 = m1.getValue();
@@ -991,7 +1240,7 @@ Iterable<Monomial<C>> {
         if (this.isZERO()) {
             return this;
         }
-        GenPolynomial<C> p = ring.getZERO().clone();
+        GenPolynomial<C> p = ring.getZERO().copy();
         SortedMap<ExpVector, C> pv = p.val;
         for (Map.Entry<ExpVector, C> m1 : val.entrySet()) {
             C c1 = m1.getValue();
@@ -1043,11 +1292,11 @@ Iterable<Monomial<C>> {
         }
         if (this instanceof GenSolvablePolynomial) {
             //throw new RuntimeException("wrong method dispatch in JRE ");
-            logger.warn("wrong method dispatch in JRE (s,e) - trying to fix");
+            logger.debug("warn: wrong method dispatch in JRE multiply(s,e) - trying to fix");
             GenSolvablePolynomial<C> T = (GenSolvablePolynomial<C>) this;
             return T.multiply(s, e);
         }
-        GenPolynomial<C> p = ring.getZERO().clone();
+        GenPolynomial<C> p = ring.getZERO().copy();
         SortedMap<ExpVector, C> pv = p.val;
         for (Map.Entry<ExpVector, C> m1 : val.entrySet()) {
             C c1 = m1.getValue();
@@ -1074,11 +1323,11 @@ Iterable<Monomial<C>> {
         }
         if (this instanceof GenSolvablePolynomial) {
             //throw new RuntimeException("wrong method dispatch in JRE ");
-            logger.warn("wrong method dispatch in JRE (e) - trying to fix");
+            logger.debug("warn: wrong method dispatch in JRE multiply(e) - trying to fix");
             GenSolvablePolynomial<C> T = (GenSolvablePolynomial<C>) this;
             return T.multiply(e);
         }
-        GenPolynomial<C> p = ring.getZERO().clone();
+        GenPolynomial<C> p = ring.getZERO().copy();
         SortedMap<ExpVector, C> pv = p.val;
         for (Map.Entry<ExpVector, C> m1 : val.entrySet()) {
             C c1 = m1.getValue();
@@ -1111,30 +1360,28 @@ Iterable<Monomial<C>> {
      */
     public GenPolynomial<C> divide(C s) {
         if (s == null || s.isZERO()) {
-            throw new ArithmeticException(this.getClass().getName() + " division by zero");
+            throw new ArithmeticException("division by zero");
         }
         if (this.isZERO()) {
             return this;
         }
         //C t = s.inverse();
         //return multiply(t);
-        GenPolynomial<C> p = ring.getZERO().clone();
+        GenPolynomial<C> p = ring.getZERO().copy();
         SortedMap<ExpVector, C> pv = p.val;
         for (Map.Entry<ExpVector, C> m : val.entrySet()) {
             ExpVector e = m.getKey();
             C c1 = m.getValue();
             C c = c1.divide(s);
-            if (false) {
+            if (debug) {
                 C x = c1.remainder(s);
                 if (!x.isZERO()) {
-                    System.out.println("divide x = " + x);
-                    throw new ArithmeticException(this.getClass().getName() + " no exact division: " + c1
-                                    + "/" + s);
+                    logger.info("divide x = " + x);
+                    throw new ArithmeticException("no exact division: " + c1 + "/" + s);
                 }
             }
             if (c.isZERO()) {
-                throw new ArithmeticException(this.getClass().getName() + " no exact division: " + c1 + "/"
-                                + s + ", in " + this);
+                throw new ArithmeticException("no exact division: " + c1 + "/" + s + ", in " + this);
             }
             pv.put(e, c); // or m1.setValue( c )
         }
@@ -1149,24 +1396,24 @@ Iterable<Monomial<C>> {
      * @param S nonzero GenPolynomial with invertible leading coefficient.
      * @return [ quotient , remainder ] with this = quotient * S + remainder and
      *         deg(remainder) &lt; deg(S) or remiander = 0.
-     * @see edu.jas.poly.PolyUtil#basePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
+     * @see edu.jas.poly.PolyUtil#baseSparsePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
      *      .
      */
     @SuppressWarnings("unchecked")
     public GenPolynomial<C>[] quotientRemainder(GenPolynomial<C> S) {
         if (S == null || S.isZERO()) {
-            throw new ArithmeticException(this.getClass().getName() + " division by zero");
+            throw new ArithmeticException("division by zero");
         }
         C c = S.leadingBaseCoefficient();
         if (!c.isUnit()) {
-            throw new ArithmeticException(this.getClass().getName() + " lbcf not invertible " + c);
+            throw new ArithmeticException("lbcf not invertible " + c);
         }
         C ci = c.inverse();
         assert (ring.nvar == S.ring.nvar);
         ExpVector e = S.leadingExpVector();
         GenPolynomial<C> h;
-        GenPolynomial<C> q = ring.getZERO().clone();
-        GenPolynomial<C> r = this.clone();
+        GenPolynomial<C> q = ring.getZERO().copy();
+        GenPolynomial<C> r = this.copy();
         while (!r.isZERO()) {
             ExpVector f = r.leadingExpVector();
             if (f.multipleOf(e)) {
@@ -1194,7 +1441,7 @@ Iterable<Monomial<C>> {
      * @param S nonzero GenPolynomial with invertible leading coefficient.
      * @return [ quotient , remainder ] with this = quotient * S + remainder and
      *         deg(remainder) &lt; deg(S) or remiander = 0.
-     * @see edu.jas.poly.PolyUtil#basePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
+     * @see edu.jas.poly.PolyUtil#baseSparsePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
      *      .
      * @deprecated use quotientRemainder()
      */
@@ -1210,7 +1457,7 @@ Iterable<Monomial<C>> {
      * over fields, but works in any case.
      * @param S nonzero GenPolynomial with invertible leading coefficient.
      * @return quotient with this = quotient * S + remainder.
-     * @see edu.jas.poly.PolyUtil#basePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
+     * @see edu.jas.poly.PolyUtil#baseSparsePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
      *      .
      */
     public GenPolynomial<C> divide(GenPolynomial<C> S) {
@@ -1224,22 +1471,22 @@ Iterable<Monomial<C>> {
      * over fields, but works in any case.
      * @param S nonzero GenPolynomial with invertible leading coefficient.
      * @return remainder with this = quotient * S + remainder.
-     * @see edu.jas.poly.PolyUtil#basePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
+     * @see edu.jas.poly.PolyUtil#baseSparsePseudoRemainder(edu.jas.poly.GenPolynomial,edu.jas.poly.GenPolynomial)
      *      .
      */
     public GenPolynomial<C> remainder(GenPolynomial<C> S) {
         if (S == null || S.isZERO()) {
-            throw new ArithmeticException(this.getClass().getName() + " division by zero");
+            throw new ArithmeticException("division by zero");
         }
         C c = S.leadingBaseCoefficient();
         if (!c.isUnit()) {
-            throw new ArithmeticException(this.getClass().getName() + " lbc not invertible " + c);
+            throw new ArithmeticException("lbc not invertible " + c);
         }
         C ci = c.inverse();
         assert (ring.nvar == S.ring.nvar);
         ExpVector e = S.leadingExpVector();
         GenPolynomial<C> h;
-        GenPolynomial<C> r = this.clone();
+        GenPolynomial<C> r = this.copy();
         while (!r.isZERO()) {
             ExpVector f = r.leadingExpVector();
             if (f.multipleOf(e)) {
@@ -1271,8 +1518,7 @@ Iterable<Monomial<C>> {
             return S;
         }
         if (ring.nvar != 1) {
-            throw new IllegalArgumentException(this.getClass().getName() + " not univariate polynomials"
-                            + ring);
+            throw new IllegalArgumentException("not univariate polynomials" + ring);
         }
         GenPolynomial<C> x;
         GenPolynomial<C> q = this;
@@ -1328,10 +1574,10 @@ Iterable<Monomial<C>> {
         GenPolynomial<C>[] qr;
         GenPolynomial<C> q = this;
         GenPolynomial<C> r = S;
-        GenPolynomial<C> c1 = ring.getONE().clone();
-        GenPolynomial<C> d1 = ring.getZERO().clone();
-        GenPolynomial<C> c2 = ring.getZERO().clone();
-        GenPolynomial<C> d2 = ring.getONE().clone();
+        GenPolynomial<C> c1 = ring.getONE().copy();
+        GenPolynomial<C> d1 = ring.getZERO().copy();
+        GenPolynomial<C> c2 = ring.getZERO().copy();
+        GenPolynomial<C> d2 = ring.getONE().copy();
         GenPolynomial<C> x1;
         GenPolynomial<C> x2;
         while (!r.isZERO()) {
@@ -1389,10 +1635,9 @@ Iterable<Monomial<C>> {
         GenPolynomial<C>[] qr;
         GenPolynomial<C> q = this;
         GenPolynomial<C> r = S;
-        GenPolynomial<C> c1 = ring.getONE().clone();
-        GenPolynomial<C> d1 = ring.getZERO().clone();
+        GenPolynomial<C> c1 = ring.getONE().copy();
+        GenPolynomial<C> d1 = ring.getZERO().copy();
         GenPolynomial<C> x1;
-        GenPolynomial<C> x2;
         while (!r.isZERO()) {
             qr = q.quotientRemainder(r);
             q = qr[0];
@@ -1417,7 +1662,7 @@ Iterable<Monomial<C>> {
 
 
     /**
-     * GenPolynomial inverse. Required by RingElem. Throws not implemented
+     * GenPolynomial inverse. Required by RingElem. Throws not invertible
      * exception.
      */
     public GenPolynomial<C> inverse() {
@@ -1425,7 +1670,7 @@ Iterable<Monomial<C>> {
             C c = leadingBaseCoefficient().inverse();
             return ring.getONE().multiply(c);
         }
-        throw new NotInvertibleException("element not invertible " + this);
+        throw new NotInvertibleException("element not invertible " + this + " :: " + ring);
     }
 
 
@@ -1462,9 +1707,9 @@ Iterable<Monomial<C>> {
      */
     public GenPolynomial<C> extend(GenPolynomialRing<C> pfac, int j, long k) {
         if (ring.equals(pfac)) { // nothing to do
-	    return this;
+            return this;
         }
-        GenPolynomial<C> Cp = pfac.getZERO().clone();
+        GenPolynomial<C> Cp = pfac.getZERO().copy();
         if (this.isZERO()) {
             return Cp;
         }
@@ -1491,9 +1736,9 @@ Iterable<Monomial<C>> {
      */
     public GenPolynomial<C> extendLower(GenPolynomialRing<C> pfac, int j, long k) {
         if (ring.equals(pfac)) { // nothing to do
-	    return this;
+            return this;
         }
-        GenPolynomial<C> Cp = pfac.getZERO().clone();
+        GenPolynomial<C> Cp = pfac.getZERO().copy();
         if (this.isZERO()) {
             return Cp;
         }
@@ -1581,7 +1826,7 @@ Iterable<Monomial<C>> {
             return pfac.getONE();
         }
         int j = pfac.nvar - 1 - i;
-        GenPolynomial<C> Cp = pfac.getZERO().clone();
+        GenPolynomial<C> Cp = pfac.getZERO().copy();
         if (this.isZERO()) {
             return Cp;
         }
@@ -1598,12 +1843,65 @@ Iterable<Monomial<C>> {
     }
 
 
+    /**
+     * Make homogeneous.
+     * @param pfac extended polynomial ring factory (by 1 variable).
+     * @return homogeneous polynomial.
+     */
+    public GenPolynomial<C> homogenize(GenPolynomialRing<C> pfac) {
+        if (ring.equals(pfac)) { // not implemented
+            throw new UnsupportedOperationException("case with same ring not implemented");
+        }
+        GenPolynomial<C> Cp = pfac.getZERO().copy();
+        if (this.isZERO()) {
+            return Cp;
+        }
+        long deg = totalDegree();
+        //int i = pfac.nvar - ring.nvar;
+        Map<ExpVector, C> C = Cp.val; //getMap();
+        Map<ExpVector, C> A = val;
+        for (Map.Entry<ExpVector, C> y : A.entrySet()) {
+            ExpVector e = y.getKey();
+            C a = y.getValue();
+            long d = deg - e.totalDeg();
+            ExpVector f = e.extend(1, 0, d);
+            C.put(f, a);
+        }
+        return Cp;
+    }
+
+
+    /**
+     * Dehomogenize.
+     * @param pfac contracted polynomial ring factory (by 1 variable).
+     * @return in homogeneous polynomial.
+     */
+    public GenPolynomial<C> deHomogenize(GenPolynomialRing<C> pfac) {
+        if (ring.equals(pfac)) { // not implemented
+            throw new UnsupportedOperationException("case with same ring not implemented");
+        }
+        GenPolynomial<C> Cp = pfac.getZERO().copy();
+        if (this.isZERO()) {
+            return Cp;
+        }
+        Map<ExpVector, C> C = Cp.val; //getMap();
+        Map<ExpVector, C> A = val;
+        for (Map.Entry<ExpVector, C> y : A.entrySet()) {
+            ExpVector e = y.getKey();
+            C a = y.getValue();
+            ExpVector f = e.contract(1, pfac.nvar);
+            C.put(f, a);
+        }
+        return Cp;
+    }
+
+
     /**
      * Reverse variables. Used e.g. in opposite rings.
      * @return polynomial with reversed variables.
      */
     public GenPolynomial<C> reverse(GenPolynomialRing<C> oring) {
-        GenPolynomial<C> Cp = oring.getZERO().clone();
+        GenPolynomial<C> Cp = oring.getZERO().copy();
         if (this.isZERO()) {
             return Cp;
         }
@@ -1662,7 +1960,7 @@ Iterable<Monomial<C>> {
      * @return new polynomial with coefficients f(this(e)).
      */
     public GenPolynomial<C> map(final UnaryFunctor<? super C, C> f) {
-        GenPolynomial<C> n = ring.getZERO().clone();
+        GenPolynomial<C> n = ring.getZERO().copy();
         SortedMap<ExpVector, C> nv = n.val;
         for (Monomial<C> m : this) {
             //logger.info("m = " + m);
