@@ -1,22 +1,21 @@
 /*
- * $Id: ModulFactory.java 1708 2008-02-24 17:28:36Z kredel $
+ * $Id: ModulFactory.java 4056 2012-07-26 17:44:13Z kredel $
  */
 
 package edu.jas.structure;
 
+
 import java.util.List;
 
 
 /**
- * Module factory interface.
- * Defines conversion from list and sparse random.
+ * Module factory interface. Defines conversion from list and sparse random.
  * @param <M> module type
  * @param <C> coefficient type
  * @author Heinz Kredel
  */
-public interface ModulFactory<M extends ModulElem<M,C>,
-                              C extends RingElem<C>> 
-                 extends AbelianGroupFactory<M> {
+public interface ModulFactory<M extends ModulElem<M, C>, C extends RingElem<C>> extends
+                AbelianGroupFactory<M> {
 
 
     /**
