@@ -1,19 +1,18 @@
 /*
- * $Id: StarRingElem.java 1708 2008-02-24 17:28:36Z kredel $
+ * $Id: StarRingElem.java 4056 2012-07-26 17:44:13Z kredel $
  */
 
 package edu.jas.structure;
 
 
 /**
- * Star ring element interface.
- * Defines norm and conjugation.
+ * Star ring element interface. Defines norm and conjugation.
  * @param <C> ring element type
  * @author Heinz Kredel
  */
 
-public interface StarRingElem<C extends StarRingElem<C>> 
-                 extends RingElem<C> {
+public interface StarRingElem<C extends StarRingElem<C>> extends RingElem<C> {
+
 
     /**
      * Conjugate of this.
