@@ -1,5 +1,5 @@
 /*
- * $Id: FactorAbstract.java 3771 2011-09-18 20:29:35Z kredel $
+ * $Id: FactorAbstract.java 4366 2013-02-03 17:56:46Z kredel $
  */
 
 package edu.jas.ufd;
@@ -7,6 +7,7 @@ package edu.jas.ufd;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Map;
 import java.util.SortedMap;
 import java.util.SortedSet;
 import java.util.TreeMap;
@@ -15,10 +16,12 @@ import java.util.TreeSet;
 import org.apache.log4j.Logger;
 
 import edu.jas.kern.TimeStatus;
+import edu.jas.poly.ExpVector;
 import edu.jas.poly.GenPolynomial;
 import edu.jas.poly.GenPolynomialRing;
 import edu.jas.poly.PolyUtil;
-import edu.jas.poly.ExpVector;
+import edu.jas.poly.OptimizedPolynomialList;
+import edu.jas.poly.TermOrderOptimization;
 import edu.jas.structure.GcdRingElem;
 import edu.jas.structure.RingFactory;
 import edu.jas.util.KsubSet;
@@ -132,11 +135,74 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
 
 
     /**
-     * GenPolynomial factorization of a squarefree polynomial, using Kronecker substitution.
+     * GenPolynomial factorization of a multivariate squarefree polynomial,
+     * using Kronecker substitution and variable order optimization.
+     * @param P squarefree and primitive! (respectively monic) multivariate
+     *            GenPolynomial over the ring C.
+     * @return [p_1,...,p_k] with P = prod_{i=1,...,r} p_i.
+     */
+    public List<GenPolynomial<C>> factorsSquarefreeOptimize(GenPolynomial<C> P) {
+        GenPolynomialRing<C> pfac = P.ring;
+        if (pfac.nvar <= 1) {
+            return baseFactorsSquarefree(P);
+        }
+        List<GenPolynomial<C>> topt = new ArrayList<GenPolynomial<C>>(1);
+        topt.add(P);
+        OptimizedPolynomialList<C> opt = TermOrderOptimization.<C> optimizeTermOrder(pfac,topt);
+        P = opt.list.get(0);
+        logger.info("optimized polynomial: " + P);
+        List<Integer> iperm = TermOrderOptimization.inversePermutation(opt.perm);
+        logger.info("optimize perm: " + opt.perm + ", de-optimize perm: " + iperm);
+
+        ExpVector degv = P.degreeVector();
+        int[] donv = degv.dependencyOnVariables();
+        List<GenPolynomial<C>> facs = null;
+        if (degv.length() == donv.length) { // all variables appear
+            logger.info("do.full factorsSquarefreeKronecker: " + P);
+            facs = factorsSquarefreeKronecker(P);
+        } else { // not all variables appear, remove unused variables
+            GenPolynomial<C> pu = PolyUtil.<C> removeUnusedUpperVariables(P);
+            //GenPolynomial<C> pl = PolyUtil.<C> removeUnusedLowerVariables(pu); // not useful after optimize
+            logger.info("do.sparse factorsSquarefreeKronecker: " + pu);
+            facs = factorsSquarefreeKronecker(pu); // pl
+            List<GenPolynomial<C>> fs = new ArrayList<GenPolynomial<C>>(facs.size());
+            GenPolynomialRing<C> pf = P.ring;
+            //GenPolynomialRing<C> pfu = pu.ring;
+            for (GenPolynomial<C> p : facs) {
+                //GenPolynomial<C> pel = p.extendLower(pfu, 0, 0L);
+                GenPolynomial<C> pe = p.extend(pf, 0, 0L); // pel
+                fs.add(pe);
+            }
+            //System.out.println("fs = " + fs);
+            facs = fs;
+        }
+        List<GenPolynomial<C>> iopt = TermOrderOptimization.<C> permutation(iperm, pfac,facs);
+        logger.info("de-optimized polynomials: " + iopt);
+        facs = normalizeFactorization(iopt);
+        return facs;
+    }
+
+
+    /**
+     * GenPolynomial factorization of a squarefree polynomial, using Kronecker
+     * substitution.
      * @param P squarefree and primitive! (respectively monic) GenPolynomial.
      * @return [p_1,...,p_k] with P = prod_{i=1,...,r} p_i.
      */
+    @Override
     public List<GenPolynomial<C>> factorsSquarefree(GenPolynomial<C> P) {
+        return factorsSquarefreeKronecker(P);
+        //return factorsSquarefreeOptimize(P);
+    }
+
+
+    /**
+     * GenPolynomial factorization of a squarefree polynomial, using Kronecker
+     * substitution.
+     * @param P squarefree and primitive! (respectively monic) GenPolynomial.
+     * @return [p_1,...,p_k] with P = prod_{i=1,...,r} p_i.
+     */
+    public List<GenPolynomial<C>> factorsSquarefreeKronecker(GenPolynomial<C> P) {
         if (P == null) {
             throw new IllegalArgumentException(this.getClass().getName() + " P != null");
         }
@@ -175,8 +241,9 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
             System.out.println("slist = " + slist);
             throw new ArithmeticException("no factorization");
         }
-        for (GenPolynomial<C> g : slist.keySet()) {
-            long e = slist.get(g);
+        for (Map.Entry<GenPolynomial<C>, Long> me : slist.entrySet()) {
+            GenPolynomial<C> g = me.getKey();
+            long e = me.getValue(); // slist.get(g);
             for (int i = 0; i < e; i++) { // is this really required? yes!
                 ulist.add(g);
             }
@@ -215,10 +282,10 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
                     System.out.print("ti(" + ti + ") ");
                     TimeStatus.checkTime(ti + " % 2000 == 0");
                 }
-                if ( !evl.multipleOf(trial.leadingExpVector()) ) {
+                if (!evl.multipleOf(trial.leadingExpVector())) {
                     continue;
                 }
-                if ( !evt.multipleOf(trial.trailingExpVector()) ) {
+                if (!evt.multipleOf(trial.trailingExpVector())) {
                     continue;
                 }
                 if (trial.degree() > deg || trial.isConstant()) {
@@ -232,7 +299,7 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
                     System.out.println("u     = " + u);
                     System.out.println("trial = " + trial);
                 }
-                GenPolynomial<C> rem = PolyUtil.<C> basePseudoRemainder(u, trial);
+                GenPolynomial<C> rem = PolyUtil.<C> baseSparsePseudoRemainder(u, trial);
                 //System.out.println(" rem = " + rem);
                 if (rem.isZERO()) {
                     logger.info("trial = " + trial);
@@ -245,16 +312,12 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
                         j = dl + 1;
                         break;
                     }
-                    //if (ulist.removeAll(flist)) {
+                    //if (ulist.removeAll(flist)) { // wrong
                     ulist = removeOnce(ulist, flist);
-                    if (ulist != null) {
-                        //System.out.println("new ulist = " + ulist);
-                        dl = (ulist.size() + 1) / 2;
-                        j = 0; // since j++
-                        break;
-                        //} else {
-                        //    logger.error("error removing flist from ulist = " + ulist);
-                    }
+                    //System.out.println("new ulist = " + ulist);
+                    dl = (ulist.size() + 1) / 2;
+                    j = 0; // since j++
+                    break;
                 }
             }
         }
@@ -268,15 +331,16 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
             //System.out.println("irred u = " + u);
             factors.add(P);
         }
-        return factors;
+        return normalizeFactorization(factors);
     }
 
 
     /**
-     * Remove one occurence of elements.
+     * Remove one occurrence of elements.
      * @param a list of objects.
      * @param b list of objects.
-     * @return remove every element of b from a, but only one occurence.
+     * @return remove every element of b from a, but only one occurrence.
+     *         <b>Note:</b> not available in java.util.
      */
     static <T> List<T> removeOnce(List<T> a, List<T> b) {
         List<T> res = new ArrayList<T>();
@@ -346,14 +410,15 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
             facs.put(P, 1L);
         }
         if (logger.isInfoEnabled()
-                && (facs.size() > 1 || (facs.size() == 1 && facs.get(facs.firstKey()) > 1))) {
+                        && (facs.size() > 1 || (facs.size() == 1 && facs.get(facs.firstKey()) > 1))) {
             logger.info("squarefree facs   = " + facs);
             //System.out.println("sfacs   = " + facs);
             //boolean tt = isFactorization(P,facs);
             //System.out.println("sfacs tt   = " + tt);
         }
-        for (GenPolynomial<C> g : facs.keySet()) {
-            Long k = facs.get(g);
+        for (Map.Entry<GenPolynomial<C>, Long> me : facs.entrySet()) {
+            GenPolynomial<C> g = me.getKey();
+            Long k = me.getValue(); //facs.get(g);
             //System.out.println("g       = " + g);
             if (pfac.coFac.isField() && !g.leadingBaseCoefficient().isONE()) {
                 g = g.monic(); // how can this happen?
@@ -476,11 +541,12 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
                 logger.info("squarefree #mfacs 1-1 = " + facs);
             }
         }
-        for (GenPolynomial<C> g : facs.keySet()) {
-            if ( g.isONE() ) { // skip 1
+        for (Map.Entry<GenPolynomial<C>, Long> me : facs.entrySet()) {
+            GenPolynomial<C> g = me.getKey();
+            if (g.isONE()) { // skip 1
                 continue;
             }
-            Long d = facs.get(g);
+            Long d = me.getValue(); // facs.get(g);
             List<GenPolynomial<C>> sfacs = factorsSquarefree(g);
             if (logger.isInfoEnabled()) {
                 logger.info("factors of squarefree ^" + d + " = " + sfacs);
@@ -571,13 +637,14 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
 
     /**
      * Degree of a factorization.
-     * @param F a  factors map [p_1 -&gt; e_1, ..., p_k -&gt; e_k].
+     * @param F a factors map [p_1 -&gt; e_1, ..., p_k -&gt; e_k].
      * @return sum_{i=1,...,k} degree(p_i)*e_i.
      */
-    public long factorsDegree(SortedMap<GenPolynomial<C>,Long> F) {
+    public long factorsDegree(SortedMap<GenPolynomial<C>, Long> F) {
         long d = 0;
-        for ( GenPolynomial<C> p : F.keySet() ) {
-	    long e = F.get(p);
+        for (Map.Entry<GenPolynomial<C>, Long> me : F.entrySet()) {
+            GenPolynomial<C> p = me.getKey();
+            long e = me.getValue(); //F.get(p);
             d += p.degree() * e;
         }
         return d;
@@ -591,7 +658,7 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
      * @return true if P = prod_{i=1,...,k} p_i**e_i , else false.
      */
     public boolean isRecursiveFactorization(GenPolynomial<GenPolynomial<C>> P,
-            SortedMap<GenPolynomial<GenPolynomial<C>>, Long> F) {
+                    SortedMap<GenPolynomial<GenPolynomial<C>>, Long> F) {
         return sengine.isRecursiveFactorization(P, F);
         // test irreducible
     }
@@ -663,7 +730,7 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
         }
         GenPolynomialRing<GenPolynomial<C>> pfac = P.ring;
         SortedMap<GenPolynomial<GenPolynomial<C>>, Long> factors = new TreeMap<GenPolynomial<GenPolynomial<C>>, Long>(
-                pfac.getComparator());
+                        pfac.getComparator());
         if (P.isZERO()) {
             return factors;
         }
@@ -693,8 +760,9 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
             r = r.multiply(ldcf);
             dfacts.put(r, E);
         }
-        for (GenPolynomial<C> f : dfacts.keySet()) {
-            Long E = dfacts.get(f);
+        for (Map.Entry<GenPolynomial<C>, Long> me : dfacts.entrySet()) {
+            GenPolynomial<C> f = me.getKey();
+            Long E = me.getValue(); //dfacts.get(f);
             GenPolynomial<GenPolynomial<C>> rp = PolyUtil.<C> recursive(pfac, f);
             factors.put(rp, E);
         }
@@ -707,27 +775,27 @@ public abstract class FactorAbstract<C extends GcdRingElem<C>> implements Factor
 
 
     /**
-     * Normalize factorization.
-     * p'_i &gt; 0 for i &gt; 1 and p'_1 != 1 if k &gt; 1.
+     * Normalize factorization. p'_i &gt; 0 for i &gt; 1 and p'_1 != 1 if k &gt;
+     * 1.
      * @param F = [p_1,...,p_k].
      * @return F' = [p'_1,...,p'_k].
      */
     public List<GenPolynomial<C>> normalizeFactorization(List<GenPolynomial<C>> F) {
-        if ( F == null || F.size() <= 1 ) {
+        if (F == null || F.size() <= 1) {
             return F;
         }
-	List<GenPolynomial<C>> Fp = new ArrayList<GenPolynomial<C>>(F.size());
+        List<GenPolynomial<C>> Fp = new ArrayList<GenPolynomial<C>>(F.size());
         GenPolynomial<C> f0 = F.get(0);
-        for ( int i = 1; i < F.size(); i++ ) {
-             GenPolynomial<C> fi = F.get(i);
-             if ( fi.signum() < 0 ) {
-                 fi = fi.negate();
-                 f0 = f0.negate();
-             }
-             Fp.add(fi);
+        for (int i = 1; i < F.size(); i++) {
+            GenPolynomial<C> fi = F.get(i);
+            if (fi.signum() < 0) {
+                fi = fi.negate();
+                f0 = f0.negate();
+            }
+            Fp.add(fi);
         }
-        if ( !f0.isONE() ) {
-            Fp.add(0,f0);
+        if (!f0.isONE()) {
+            Fp.add(0, f0);
         }
         return Fp;
     }
