@@ -1,5 +1,5 @@
 /*
- * $Id: FactorInteger.java 3776 2011-09-23 20:08:13Z kredel $
+ * $Id: FactorInteger.java 4343 2012-12-31 17:10:56Z kredel $
  */
 
 package edu.jas.ufd;
@@ -22,8 +22,8 @@ import edu.jas.arith.PrimeList;
 import edu.jas.poly.ExpVector;
 import edu.jas.poly.GenPolynomial;
 import edu.jas.poly.GenPolynomialRing;
-import edu.jas.poly.PolyUtil;
 import edu.jas.poly.OptimizedPolynomialList;
+import edu.jas.poly.PolyUtil;
 import edu.jas.poly.TermOrderOptimization;
 import edu.jas.structure.GcdRingElem;
 import edu.jas.structure.Power;
@@ -49,7 +49,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
     private static final Logger logger = Logger.getLogger(FactorInteger.class);
 
 
-    private final boolean debug = true || logger.isDebugEnabled();
+    private final boolean debug = logger.isDebugEnabled();
 
 
     /**
@@ -261,7 +261,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
                 if (debug) {
                     logger.info("lifting from " + mlist);
                 }
-                if (false && P.leadingBaseCoefficient().isONE()) {
+                if (P.leadingBaseCoefficient().isONE()) { // monic case
                     factors = searchFactorsMonic(P, M, mlist, AD); // does now work in all cases
                     if (factors.size() == 1) {
                         factors = searchFactorsNonMonic(P, M, mlist, AD);
@@ -283,16 +283,14 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
                     factors = searchFactorsMonic(P, M, mlist, AD); // does now work in all cases
                     t = System.currentTimeMillis() - t;
                     //System.out.println("monic time = " + t);
-                    if (false && debug) {
+                    if (debug) {
                         t = System.currentTimeMillis();
                         List<GenPolynomial<BigInteger>> fnm = searchFactorsNonMonic(P, M, mlist, AD);
                         t = System.currentTimeMillis() - t;
                         System.out.println("non monic time = " + t);
-                        if (debug) {
-                            if (!factors.equals(fnm)) {
-                                System.out.println("monic factors     = " + factors);
-                                System.out.println("non monic factors = " + fnm);
-                            }
+                        if (!factors.equals(fnm)) {
+                            System.out.println("monic factors     = " + factors);
+                            System.out.println("non monic factors = " + fnm);
                         }
                     }
                 } catch (RuntimeException e) {
@@ -459,7 +457,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
                 //trial = engine.basePrimitivePart( trial.multiply(ldcf) );
                 trial = engine.basePrimitivePart(trial);
                 //System.out.println("pp(trial)= " + trial);
-                if (PolyUtil.<BigInteger> basePseudoRemainder(u, trial).isZERO()) {
+                if (PolyUtil.<BigInteger> baseSparsePseudoRemainder(u, trial).isZERO()) {
                     logger.info("successful trial = " + trial);
                     //System.out.println("trial    = " + trial);
                     //System.out.println("flist    = " + flist);
@@ -589,7 +587,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
 
                 itrial = engine.basePrimitivePart(itrial);
                 //System.out.println("pp(trial)= " + itrial);
-                if (PolyUtil.<BigInteger> basePseudoRemainder(u, itrial).isZERO()) {
+                if (PolyUtil.<BigInteger> baseSparsePseudoRemainder(u, itrial).isZERO()) {
                     logger.info("successful trial = " + itrial);
                     //System.out.println("trial    = " + itrial);
                     //System.out.println("cofactor = " + icofactor);
@@ -637,12 +635,13 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
     @Override
     public List<GenPolynomial<BigInteger>> factorsSquarefree(GenPolynomial<BigInteger> P) {
         GenPolynomialRing<BigInteger> pfac = P.ring;
-        if ( pfac.nvar <= 1 ) {
+        if (pfac.nvar <= 1) {
             return baseFactorsSquarefree(P);
         }
         List<GenPolynomial<BigInteger>> topt = new ArrayList<GenPolynomial<BigInteger>>(1);
         topt.add(P);
-        OptimizedPolynomialList<BigInteger> opt = TermOrderOptimization.<BigInteger> optimizeTermOrder(pfac,topt);
+        OptimizedPolynomialList<BigInteger> opt = TermOrderOptimization.<BigInteger> optimizeTermOrder(pfac,
+                        topt);
         P = opt.list.get(0);
         logger.info("optimized polynomial: " + P);
         List<Integer> iperm = TermOrderOptimization.inversePermutation(opt.perm);
@@ -680,14 +679,14 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
                 //e.printStackTrace();
             }
         }
-        if (facs != null) {
-            List<GenPolynomial<BigInteger>> iopt = TermOrderOptimization. <BigInteger> permutation(iperm,pfac,facs);
-            logger.info("de-optimized polynomials: " + iopt);
-            facs = normalizeFactorization(iopt);
-        } else {
+        if (facs == null) {
             logger.info("factorsSquarefreeHensel not applicable or failed, reverting to Kronecker for: " + P);
             facs = super.factorsSquarefree(P);
         }
+        List<GenPolynomial<BigInteger>> iopt = TermOrderOptimization.<BigInteger> permutation(iperm, pfac,
+                        facs);
+        logger.info("de-optimized polynomials: " + iopt);
+        facs = normalizeFactorization(iopt);
         return facs;
     }
 
@@ -787,9 +786,11 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
         boolean isPrimitive = true;
         boolean notLucky = true;
         while (notLucky) { // for Wang's test
-            if (Math.abs(evStart) > 400L) {
-                System.out.println("P = " + P + ", lprr = " + lprr + ", lfacs = " + lfacs);
-                throw new RuntimeException("no lucky evaluation point found after " + evStart + " iterations");
+            if (Math.abs(evStart) > 371L) {
+                logger.warn("no lucky evaluation point for: P = " + P + ", lprr = " + lprr + ", lfacs = "
+                                + lfacs);
+                throw new RuntimeException("no lucky evaluation point found after " + Math.abs(evStart)
+                                + " iterations");
             }
             if (Math.abs(evStart) % 100L <= 3L) {
                 ran = ran * (Math.PI - 2.14);
@@ -949,13 +950,13 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
             // determine leading coefficient polynomials for factors
             lf = new ArrayList<GenPolynomial<BigInteger>>();
             lpx = lprr.ring.getONE();
-            for (GenPolynomial<BigInteger> pp : ufactors) {
+            for (GenPolynomial<BigInteger> unused : ufactors) {
                 lf.add(lprr.ring.getONE());
             }
             //System.out.println("lf = " + lf);             
             if (!isMonic || !pecw.isONE()) {
                 if (lfacs.size() > 0 && lfacs.get(0).isConstant()) {
-                    GenPolynomial<BigInteger> xx = lfacs.remove(0);
+                    GenPolynomial<BigInteger> unused = lfacs.remove(0);
                     //BigInteger xxi = xx.leadingBaseCoefficient();
                     //System.out.println("xx = " + xx + " == ped = " +ped);
                 }
@@ -1033,6 +1034,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
                         }
                     }
                     if (!lf.equals(ln) || !un.equals(ufactors)) {
+                        logger.debug("!lf.equals(ln) || !un.equals(ufactors)");
                         //System.out.println("pe  = " + pe);
                         //System.out.println("#ln  = " + ln + ", #lf = " + lf);
                         //System.out.println("#un  = " + un + ", #ufactors = " + ufactors);
@@ -1093,7 +1095,11 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
                     tp = new TrialParts(V, pes, un, cei, ln);
                 }
                 //System.out.println("trialParts = " + tp);
-                tParts.add(tp);
+                if (tp.univPoly != null) {
+                    if (tp.ldcfEval.size() != 0) {
+                        tParts.add(tp);
+                    }
+                }
                 if (tParts.size() < trials) {
                     notLucky = true;
                 }
@@ -1125,6 +1131,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
         ufactors = tpmin.univFactors;
         cei = tpmin.ldcfEval; // unused
         lf = tpmin.ldcfFactors;
+        logger.info("iterations    = " + Math.abs(evStart));
         logger.info("minimal trial = " + tpmin);
 
         GenPolynomialRing<BigInteger> ufac = pe.ring;
@@ -1229,17 +1236,17 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
         //System.out.println("pd = " + pd);
         logger.info("multivariate modulo p^k: " + pq);
 
-        List<MOD> Vm = new ArrayList<MOD>(V.size());
-        for (BigInteger v : V) {
-            MOD vm = muqfac.fromInteger(v.getVal());
-            Vm.add(vm);
-        }
+        //List<MOD> Vm = new ArrayList<MOD>(V.size());
+        //for (BigInteger v : V) {
+        //    MOD vm = muqfac.fromInteger(v.getVal());
+        //    Vm.add(vm);
+        //}
         //System.out.println("Vm = " + Vm);
 
         // Hensel lifting of factors
         List<GenPolynomial<MOD>> mlift;
         try {
-            mlift = HenselMultUtil.<MOD> liftHensel(pd, pq, muqfactors, Vm, k, lf);
+            mlift = HenselMultUtil.<MOD> liftHensel(pd, pq, muqfactors, V, k, lf);
             logger.info("mlift = " + mlift);
         } catch (NoLiftingException nle) {
             //System.out.println("exception : " + nle);
@@ -1286,7 +1293,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
                 if (debug) {
                     logger.info("trial    = " + trial); // + ", mtrial = " + mtrial);
                 }
-                if (PolyUtil.<BigInteger> basePseudoRemainder(ui, trial).isZERO()) {
+                if (PolyUtil.<BigInteger> baseSparsePseudoRemainder(ui, trial).isZERO()) {
                     logger.info("successful trial = " + trial);
                     factors.add(trial);
                     ui = PolyUtil.<BigInteger> basePseudoDivide(ui, trial);
@@ -1353,7 +1360,7 @@ public class FactorInteger<MOD extends GcdRingElem<MOD> & Modular> extends Facto
     boolean isNearlySquarefree(GenPolynomial<BigInteger> P) { // unused
         // in main variable
         GenPolynomialRing<BigInteger> pfac = P.ring;
-        if (true || pfac.nvar <= 4) {
+        if (pfac.nvar >= 0) { // allways true
             return sengine.isSquarefree(P);
         }
         GenPolynomialRing<GenPolynomial<BigInteger>> rfac = pfac.recursive(1);
