@@ -1,5 +1,5 @@
 /*
- * $Id: IdealTest.java 3789 2011-10-01 18:54:43Z kredel $
+ * $Id: IdealTest.java 4316 2012-12-15 19:29:42Z kredel $
  */
 
 package edu.jas.application;
@@ -7,7 +7,6 @@ package edu.jas.application;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Arrays;
 
 import junit.framework.Test;
 import junit.framework.TestCase;
@@ -41,6 +40,7 @@ public class IdealTest extends TestCase {
 
     private static final Logger logger = Logger.getLogger(IdealTest.class);
 
+
     /**
      * main
      */
@@ -307,12 +307,12 @@ public class IdealTest extends TestCase {
         assertTrue("I contains(H)", I.contains(H));
         assertTrue("J contains(H)", J.contains(H));
         assertTrue("H contains(K)", H.contains(K));
-        if (false /*! H.equals(K)*/) {
-            System.out.println("I = " + I);
-            System.out.println("J = " + J);
-            System.out.println("K = " + K);
-            System.out.println("H = " + H);
-        }
+        //if (false /*! H.equals(K)*/) {
+        //    System.out.println("I = " + I);
+        //    System.out.println("J = " + J);
+        //    System.out.println("K = " + K);
+        //    System.out.println("H = " + H);
+        //}
 
 
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -339,12 +339,12 @@ public class IdealTest extends TestCase {
         assertTrue("I contains(H)", I.contains(H));
         assertTrue("J contains(H)", J.contains(H));
         assertTrue("H contains(K)", H.contains(K));
-        if (false /*! H.equals(K)*/) {
-            System.out.println("I = " + I);
-            System.out.println("J = " + J);
-            System.out.println("K = " + K);
-            System.out.println("H = " + H);
-        }
+        //if (false /*! H.equals(K)*/) {
+        //    System.out.println("I = " + I);
+        //    System.out.println("J = " + J);
+        //    System.out.println("K = " + K);
+        //    System.out.println("H = " + H);
+        //}
 
 
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -371,12 +371,12 @@ public class IdealTest extends TestCase {
         assertTrue("I contains(H)", I.contains(H));
         assertTrue("J contains(H)", J.contains(H));
         assertTrue("H contains(K)", H.contains(K));
-        if (false /*! H.equals(K)*/) {
-            System.out.println("I = " + I);
-            System.out.println("J = " + J);
-            System.out.println("K = " + K);
-            System.out.println("H = " + H);
-        }
+        //if (false /*! H.equals(K)*/) {
+        //    System.out.println("I = " + I);
+        //    System.out.println("J = " + J);
+        //    System.out.println("K = " + K);
+        //    System.out.println("H = " + H);
+        //}
     }
 
 
@@ -484,13 +484,6 @@ public class IdealTest extends TestCase {
         assertTrue("not isZERO( H )", !H.isZERO());
         assertTrue("isGB( H )", H.isGB());
         assertTrue("equals(H,I)", H.equals(I)); // GBs only
-
-        if (false) {
-            System.out.println("I = " + I);
-            System.out.println("J = " + J);
-            System.out.println("K = " + K);
-            System.out.println("H = " + H);
-        }
     }
 
 
@@ -671,9 +664,9 @@ public class IdealTest extends TestCase {
     public void testIdealRadicalMember() {
 
         Ideal<BigRational> I;
-        Ideal<BigRational> J;
-        Ideal<BigRational> K;
-        Ideal<BigRational> JJ;
+        //Ideal<BigRational> J;
+        //Ideal<BigRational> K;
+        //Ideal<BigRational> JJ;
 
         a = fac.random(kl - 1, ll, el - 1, q);
         b = fac.random(kl - 1, ll, el, q);
@@ -870,7 +863,7 @@ public class IdealTest extends TestCase {
         assertTrue("isGB( I )", I.isGB());
         //System.out.println("I = " + I);
 
-        J = I.clone(); //new Ideal<BigRational>(fac,L);
+        J = I.copy(); //new Ideal<BigRational>(fac,L);
         J.doToptimize();
         assertTrue("not isZERO( J )", !J.isZERO());
         assertTrue("isGB( J )", J.isGB());
@@ -884,7 +877,7 @@ public class IdealTest extends TestCase {
         L.add(b);
 
         I = new Ideal<BigRational>(fac, L);
-        K = I.clone();
+        K = I.copy();
         I.doGB();
         assertTrue("not isZERO( I )", !I.isZERO());
         assertTrue("isGB( I )", I.isGB());
@@ -896,7 +889,7 @@ public class IdealTest extends TestCase {
         assertTrue("isGB( K )", K.isGB());
         //System.out.println("GB(opt(K)) = " + K);
 
-        J = I.clone();
+        J = I.copy();
         J.doToptimize();
         assertTrue("not isZERO( J )", !J.isZERO());
         assertTrue("isGB( J )", J.isGB());
@@ -910,7 +903,7 @@ public class IdealTest extends TestCase {
         L.add(c);
 
         I = new Ideal<BigRational>(fac, L);
-        K = I.clone();
+        K = I.copy();
         I.doGB();
         assertTrue("not isZERO( I )", !I.isZERO());
         assertTrue("isGB( I )", I.isGB());
@@ -922,7 +915,7 @@ public class IdealTest extends TestCase {
         assertTrue("isGB( K )", K.isGB());
         //System.out.println("GB(opt(K)) = " + K);
 
-        J = I.clone();
+        J = I.copy();
         J.doToptimize();
         assertTrue("not isZERO( J )", !J.isZERO());
         assertTrue("isGB( J )", J.isGB());
@@ -1005,6 +998,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1058,6 +1052,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1092,8 +1087,7 @@ public class IdealTest extends TestCase {
         GenPolynomialRing<Complex<BigDecimal>> dcfac = new GenPolynomialRing<Complex<BigDecimal>>(dcc, dfac);
         //System.out.println("dcfac = " + dcfac);
 
-        List<List<Complex<BigDecimal>>> roots = PolyUtilApp.<BigRational, BigRational> complexRootTuples(I,
-                eps);
+        List<List<Complex<BigDecimal>>> roots = PolyUtilApp.<BigRational> complexRootTuples(I, eps);
         //System.out.println("roots = " + roots + "\n");
         for (GenPolynomial<BigRational> p : I.getList()) {
             GenPolynomial<BigDecimal> dp = PolyUtil.<BigRational> decimalFromRational(dfac, p);
@@ -1126,6 +1120,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1149,11 +1144,12 @@ public class IdealTest extends TestCase {
 
         BigRational eps = new BigRational(1, 1000000);
         eps = eps.multiply(eps);
-        eps = eps.multiply(eps).multiply(eps);
+        eps = eps.multiply(eps); //.multiply(eps);
         BigDecimal e = new BigDecimal(eps.getRational());
         e = e.abs(); //.multiply(e);
+        eps = eps.multiply(new BigRational(1, 100));
 
-        List<List<BigDecimal>> roots = PolyUtilApp.<BigRational, BigRational> realRootTuples(I, eps);
+        List<List<BigDecimal>> roots = PolyUtilApp.<BigRational> realRootTuples(I, eps);
         //System.out.println("roots = " + roots + "\n");
         // polynomials with decimal coefficients
         BigDecimal dc = BigDecimal.ONE;
@@ -1167,7 +1163,7 @@ public class IdealTest extends TestCase {
                 BigDecimal ev = PolyUtil.<BigDecimal> evaluateAll(dc, dfac, dp, r);
                 if (ev.abs().compareTo(e) > 0) {
                     //System.out.println("ev = " + ev);
-                    fail("ev > eps : " + ev + " > " + e);
+                    fail("ev > e : " + ev + " > " + e + ", eps = " + new BigDecimal(eps));
                 }
             }
         }
@@ -1189,6 +1185,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1234,6 +1231,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1261,13 +1259,13 @@ public class IdealTest extends TestCase {
 
         BigRational eps = new BigRational(1, 1000000);
         eps = eps.multiply(eps);
-        eps = eps.multiply(eps).multiply(eps);
+        eps = eps.multiply(eps); //.multiply(eps);
         BigDecimal e = new BigDecimal(eps.getRational());
         e = e.abs(); //.multiply(e);
+        eps = eps.multiply(new BigRational(1, 10));
         BigDecimal dc = BigDecimal.ONE;
 
-        List<IdealWithRealRoots<BigRational>> roots 
-             = PolyUtilApp.<BigRational, BigRational> realRoots(zd,eps);
+        List<IdealWithRealRoots<BigRational>> roots = PolyUtilApp.<BigRational> realRoots(zd, eps);
         //System.out.println("roots = " + roots + "\n");
 
         for (IdealWithRealRoots<BigRational> Ir : roots) {
@@ -1297,12 +1295,13 @@ public class IdealTest extends TestCase {
         BigRational coeff = new BigRational(1, 1);
         to = new TermOrder(TermOrder.INVLEX);
         //vars = new String[] { "x", "y" };
-        vars = new String[] { "x", "y" , "z" };
+        vars = new String[] { "x", "y", "z" };
         fac = new GenPolynomialRing<BigRational>(coeff, vars.length, to, vars);
 
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1315,7 +1314,7 @@ public class IdealTest extends TestCase {
         //b = fac.parse("( y^2 + 4 )");
         c = fac.parse("( z^2 - x y )");
 
-        if (a.isZERO() || b.isZERO() ) {
+        if (a.isZERO() || b.isZERO()) {
             return;
         }
 
@@ -1338,18 +1337,18 @@ public class IdealTest extends TestCase {
         BigDecimal e = new BigDecimal(eps.getRational());
         e = e.abs(); //.multiply(e);
 
-        List<IdealWithComplexAlgebraicRoots<BigRational,BigRational>> roots 
-            = PolyUtilApp.<BigRational, BigRational> complexAlgebraicRoots(zd); //, eps);
+        List<IdealWithComplexAlgebraicRoots<BigRational>> roots = PolyUtilApp
+                        .<BigRational> complexAlgebraicRoots(zd); //, eps);
         //System.out.println("roots = " + roots + "\n");
 
         ComplexRing<BigDecimal> dcc = new ComplexRing<BigDecimal>(e);
 
         int d = 0;
         int s = 0;
-        for (IdealWithComplexAlgebraicRoots<BigRational,BigRational> Ic : roots) {
+        for (IdealWithComplexAlgebraicRoots<BigRational> Ic : roots) {
             List<GenPolynomial<BigRational>> L = Ic.ideal.getList();
-            List<GenPolynomial<Complex<BigDecimal>>> Ld 
-                = new ArrayList<GenPolynomial<Complex<BigDecimal>>>(L.size());
+            List<GenPolynomial<Complex<BigDecimal>>> Ld = new ArrayList<GenPolynomial<Complex<BigDecimal>>>(
+                            L.size());
             s += Ic.can.size();
 
             GenPolynomialRing<BigDecimal> dfac = new GenPolynomialRing<BigDecimal>(e, Ic.ideal.list.ring);
@@ -1369,8 +1368,8 @@ public class IdealTest extends TestCase {
             }
             d += ds;
             List<List<Complex<BigDecimal>>> droot = Ic.decimalApproximation();
-            for ( List<Complex<BigDecimal>> dr : droot ) {
-                //System.out.println("dr = " + dr);
+            for (List<Complex<BigDecimal>> dr : droot) {
+                //System.out.println("dr = " + dr); // TODO
             }
         }
         logger.info("#roots = " + s + ", #vr-dim = " + d);
@@ -1392,6 +1391,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1461,6 +1461,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1503,6 +1504,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1546,6 +1548,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1589,6 +1592,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1610,19 +1614,19 @@ public class IdealTest extends TestCase {
         assertTrue("isGB( I )", I.isGB());
         //System.out.println("I = " + I);
 
-        List<IdealWithRealAlgebraicRoots<BigRational, BigRational>> iur;
-        iur = PolyUtilApp.<BigRational, BigRational> realAlgebraicRoots(I);
+        List<IdealWithRealAlgebraicRoots<BigRational>> iur;
+        iur = PolyUtilApp.<BigRational> realAlgebraicRoots(I);
 
         List<IdealWithUniv<BigRational>> iul = new ArrayList<IdealWithUniv<BigRational>>();
-        for (IdealWithRealAlgebraicRoots<BigRational, BigRational> iu : iur) {
-            iul.add((IdealWithUniv<BigRational>) iu);
+        for (IdealWithRealAlgebraicRoots<BigRational> iu : iur) {
+            iul.add(iu);
         }
         assertTrue("is contained in intersection ", I.isZeroDimDecomposition(iul));
 
-        for (IdealWithRealAlgebraicRoots<BigRational, BigRational> iu : iur) {
+        for (IdealWithRealAlgebraicRoots<BigRational> iu : iur) {
             //System.out.println("iu = " + iu);
             //System.out.println("");
-            List<List<BigDecimal>> rd = iu.decimalApproximation();
+            List<List<BigDecimal>> rd = iu.decimalApproximation(); // TODO
             //System.out.println("iu = " + iu);
             //System.out.println("");
         }
@@ -1643,6 +1647,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1699,6 +1704,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1765,6 +1771,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1835,6 +1842,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
@@ -1905,6 +1913,7 @@ public class IdealTest extends TestCase {
         vars = fac.getVars();
         //System.out.println("vars = " + Arrays.toString(vars));
         //System.out.println("fac = " + fac);
+        assertTrue("vars.length == 3 ", vars.length == 3);
 
         Ideal<BigRational> I;
         L = new ArrayList<GenPolynomial<BigRational>>();
