@@ -1,5 +1,5 @@
 /*
- * $Id: PolyUtilTest.java 3789 2011-10-01 18:54:43Z kredel $
+ * $Id: PolyUtilTest.java 4015 2012-07-22 12:05:38Z kredel $
  */
 
 package edu.jas.poly;
@@ -20,6 +20,11 @@ import edu.jas.arith.ModIntegerRing;
 import edu.jas.arith.Product;
 import edu.jas.arith.ProductRing;
 
+import edu.jas.kern.ComputerThreads;
+import edu.jas.ufd.PolyUfdUtil;
+import edu.jas.ufd.Quotient;
+import edu.jas.ufd.QuotientRing;
+
 
 /**
  * PolyUtil tests with JUnit.
@@ -47,7 +52,7 @@ public class PolyUtilTest extends TestCase {
 
 
     /**
- */
+     */
     public static Test suite() {
         TestSuite suite = new TestSuite(PolyUtilTest.class);
         return suite;
@@ -176,7 +181,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test recursive <--> distributive conversion.
-     * 
      */
     public void testConversion() {
         c = dfac.getONE();
@@ -201,7 +205,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test recursive <--> distributive ring conversion.
-     * 
      */
     public void testConversionRing() {
         GenPolynomialRing<GenPolynomial<BigInteger>> rf = dfac.recursive(1);
@@ -215,7 +218,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test random recursive <--> distributive conversion.
-     * 
      */
     public void testRandomConversion() {
         for (int i = 0; i < 7; i++) {
@@ -238,7 +240,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test random rational <--> integer conversion.
-     * 
      */
     public void testRationalConversion() {
         GenPolynomialRing<BigRational> rfac = new GenPolynomialRing<BigRational>(new BigRational(1), rl, to);
@@ -270,7 +271,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test random modular <--> integer conversion.
-     * 
      */
     public void testModularConversion() {
         ModIntegerRing pm = new ModIntegerRing(getPrime1());
@@ -300,7 +300,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test chinese remainder.
-     * 
      */
     public void testChineseRemainder() {
         java.math.BigInteger p1 = getPrime1();
@@ -366,7 +365,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test complex conversion.
-     * 
      */
     public void testComplexConversion() {
         BigRational rf = new BigRational(1);
@@ -411,9 +409,179 @@ public class PolyUtilTest extends TestCase {
     }
 
 
+    /**
+     * Test base pseudo division.
+     */
+    public void testBasePseudoDivision() {
+        String[] names = new String[] { "x" };
+        dfac = new GenPolynomialRing<BigInteger>(new BigInteger(1),to,names);
+        GenPolynomialRing<BigRational> rdfac = new GenPolynomialRing<BigRational>(new BigRational(1),dfac);
+        //System.out.println("\ndfac  = " + dfac);
+        //System.out.println("rdfac = " + rdfac);
+
+        a = dfac.random(kl, 2*ll, el+17, q);
+        //a = dfac.parse(" 3 x^5 + 44 ");
+        //b = a;
+        b = dfac.random(kl, 2*ll, el+3, q);
+        //a = a.multiply(b);
+        //a = a.sum(b);
+        //b = dfac.parse(" 2 x^2 + 40 ");
+        //System.out.println("a   = " + a);
+        //System.out.println("b   = " + b);
+
+        GenPolynomial<BigInteger>[] QR = PolyUtil.<BigInteger> basePseudoQuotientRemainder(a, b);
+        c = QR[1];
+        d = QR[0];
+        //System.out.println("q   = " + d);
+        //System.out.println("r   = " + c);
+
+        boolean t = PolyUtil.<BigInteger> isBasePseudoQuotientRemainder(a, b, d, c);
+        assertTrue("lc^n a = q b + r: " + c, t);
+
+        GenPolynomial<BigRational> ap = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,a);
+        GenPolynomial<BigRational> bp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,b);
+        GenPolynomial<BigRational> cp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,c);
+        GenPolynomial<BigRational> dp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,d);
+        //System.out.println("ap  = " + ap);
+        //System.out.println("bp  = " + bp);
+        //System.out.println("cp  = " + cp);
+        ////System.out.println("dp  = " + dp);
+        //System.out.println("dp  = " + dp.monic());
+
+        GenPolynomial<BigRational> qp = ap.divide(bp);
+        GenPolynomial<BigRational> rp = ap.remainder(bp);
+        //System.out.println("qp  = " + qp);
+        //System.out.println("qp  = " + qp.monic());
+        //System.out.println("rp  = " + rp);
+        GenPolynomial<BigRational> rhs = qp.multiply(bp).sum(rp);
+        //System.out.println("qp bp + rp  = " + rhs);
+
+        assertEquals("ap = qp bp + rp: ", ap, rhs);
+
+        assertEquals("cp = rp: ", rp.monic(), cp.monic() );
+        assertEquals("dp = qp: ", qp.monic(), dp.monic() ); // ??
+        //System.out.println("dp = qp: " + qp.monic().equals(dp.monic()) );
+    }
+
+
+    /**
+     * Test base sparse pseudo division.
+     */
+    public void testBasePseudoDivisionSparse() {
+        String[] names = new String[] { "x" };
+        dfac = new GenPolynomialRing<BigInteger>(new BigInteger(1),to,names);
+        GenPolynomialRing<BigRational> rdfac = new GenPolynomialRing<BigRational>(new BigRational(1),dfac);
+        //System.out.println("\ndfac  = " + dfac);
+        //System.out.println("rdfac = " + rdfac);
+
+        a = dfac.random(kl, 2*ll, el+17, q);
+        //a = dfac.parse(" 3 x^5 + 44 ");
+        //b = a;
+        b = dfac.random(kl, 2*ll, el+3, q);
+        //a = a.multiply(b);
+        //a = a.sum(b);
+        //b = dfac.parse(" 2 x^2 + 40 ");
+        //System.out.println("a   = " + a);
+        //System.out.println("b   = " + b);
+
+        d = PolyUtil.<BigInteger> basePseudoDivide(a, b);
+        //System.out.println("q   = " + d);
+        c = PolyUtil.<BigInteger> baseSparsePseudoRemainder(a, b);
+        //System.out.println("r   = " + c);
+
+        boolean t = PolyUtil.<BigInteger> isBasePseudoQuotientRemainder(a, b, d, c);
+        assertTrue("lc^n a = q b + r: " + c, t);
+
+        GenPolynomial<BigRational> ap = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,a);
+        GenPolynomial<BigRational> bp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,b);
+        GenPolynomial<BigRational> cp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,c);
+        GenPolynomial<BigRational> dp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,d);
+        //System.out.println("ap  = " + ap);
+        //System.out.println("bp  = " + bp);
+        //System.out.println("cp  = " + cp);
+        ////System.out.println("dp  = " + dp);
+        //System.out.println("dp  = " + dp.monic());
+
+        GenPolynomial<BigRational> qp = ap.divide(bp);
+        GenPolynomial<BigRational> rp = ap.remainder(bp);
+        //System.out.println("qp  = " + qp);
+        //System.out.println("qp  = " + qp.monic());
+        //System.out.println("rp  = " + rp);
+        GenPolynomial<BigRational> rhs = qp.multiply(bp).sum(rp);
+        //System.out.println("qp bp + rp  = " + rhs);
+
+        assertEquals("ap = qp bp + rp: ", ap, rhs);
+
+        assertEquals("cp = rp: ", rp.monic(), cp.monic() );
+        assertEquals("dp = qp: ", qp.monic(), dp.monic() ); // ??
+        //System.out.println("dp = qp: " + qp.monic().equals(dp.monic()) );
+    }
+
+
+    /**
+     * Test base dense pseudo division.
+     */
+    public void testBasePseudoDivisionDense() {
+        String[] names = new String[] { "x" };
+        dfac = new GenPolynomialRing<BigInteger>(new BigInteger(1),to,names);
+        GenPolynomialRing<BigRational> rdfac = new GenPolynomialRing<BigRational>(new BigRational(1),dfac);
+        //System.out.println("\ndfac  = " + dfac);
+        //System.out.println("rdfac = " + rdfac);
+
+        a = dfac.random(kl, 2*ll, el+17, q);
+        //a = dfac.parse(" 3 x^5 + 44 ");
+        //b = a;
+        b = dfac.random(kl, 2*ll, el+3, q);
+        //a = a.multiply(b);
+        //a = a.sum(b);
+        //b = dfac.parse(" 2 x^2 + 40 ");
+        //System.out.println("a   = " + a);
+        //System.out.println("b   = " + b);
+
+        d = PolyUtil.<BigInteger> baseDensePseudoQuotient(a, b);
+        //System.out.println("q   = " + d);
+        c = PolyUtil.<BigInteger> baseDensePseudoRemainder(a, b);
+        //System.out.println("r   = " + c);
+
+        boolean t = PolyUtil.<BigInteger> isBasePseudoQuotientRemainder(a, b, d, c);
+        assertTrue("lc^n a = q b + r: " + c, t);
+
+        GenPolynomial<BigRational> ap = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,a);
+        GenPolynomial<BigRational> bp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,b);
+        GenPolynomial<BigRational> cp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,c);
+        GenPolynomial<BigRational> dp = PolyUtil.<BigRational> fromIntegerCoefficients(rdfac,d);
+        //System.out.println("ap  = " + ap);
+        //System.out.println("bp  = " + bp);
+        //System.out.println("cp  = " + cp);
+        ////System.out.println("dp  = " + dp);
+        //System.out.println("dp  = " + dp.monic());
+
+        GenPolynomial<BigRational> qp = ap.divide(bp);
+        GenPolynomial<BigRational> rp = ap.remainder(bp);
+        //System.out.println("qp  = " + qp);
+        //System.out.println("qp  = " + qp.monic());
+        //System.out.println("rp  = " + rp);
+        GenPolynomial<BigRational> rhs = qp.multiply(bp).sum(rp);
+        //System.out.println("qp bp + rp  = " + rhs);
+
+        assertEquals("ap = qp bp + rp: ", ap, rhs);
+
+        assertEquals("cp = rp: ", rp.monic(), cp.monic() );
+        assertEquals("dp = qp: ", qp.monic(), dp.monic() ); // ??
+        //System.out.println("dp = qp: " + qp.monic().equals(dp.monic()) );
+    }
+
+
+    /**
+     * Test recursive pseudo division.
+     * @see edu.jas.ufd.PolyUfdUtilTest#testRecursivePseudoDivisionSparse
+     */
+    public void testRecursivePseudoDivision() {
+    }
+
+
     /**
      * Test evaluate main recursive.
-     * 
      */
     public void testEvalMainRecursive() {
         ai = (new BigInteger()).random(kl);
@@ -478,7 +646,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test evaluate main.
-     * 
      */
     public void testEvalMain() {
         ei = (new BigInteger()).random(kl);
@@ -525,7 +692,6 @@ public class PolyUtilTest extends TestCase {
 
         assertEquals("eval(a+b) == eval(a) + eval(b)", ci, di);
 
-
         c = b.multiply(a);
         //System.out.println("c  = " + c);
 
@@ -545,7 +711,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test evaluate first.
-     * 
      */
     public void testEvalFirst() {
         ei = (new BigInteger()).random(kl);
@@ -601,7 +766,6 @@ public class PolyUtilTest extends TestCase {
 
         assertEquals("eval(a+b) == eval(a) + eval(b)", ce, de);
 
-
         c = b.multiply(a);
         //System.out.println("c  = " + c);
 
@@ -621,7 +785,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test evaluate all.
-     * 
      */
     public void testEvalAll() {
         BigInteger cfac = new BigInteger();
@@ -694,7 +857,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test interpolate univariate 1 polynomial.
-     * 
      */
     public void testInterpolateUnivariateOne() {
         ModInteger ai, bi, ci, di, ei, fi, gi, hi;
@@ -710,11 +872,9 @@ public class PolyUtilTest extends TestCase {
         cfac = new GenPolynomialRing<ModInteger>(fac, 1, to);
         //System.out.println("cfac  = " + cfac);
 
-
         a = cfac.getONE();
         //System.out.println("a  = " + a);
 
-
         ei = fac.fromInteger(11);
         //System.out.println("ei  = " + ei);
         // a(ei)
@@ -729,7 +889,6 @@ public class PolyUtilTest extends TestCase {
         //System.out.println("bi   = " + bi);
         assertTrue("isONE( bi )", bi.isONE());
 
-
         // interpolation result
         r = cfac.getZERO();
         //System.out.println("r   = " + r);
@@ -738,7 +897,6 @@ public class PolyUtilTest extends TestCase {
         Q = cfac.getONE();
         //System.out.println("Q   = " + Q);
 
-
         ci = PolyUtil.<ModInteger> evaluateMain(fac, Q, ei);
         //System.out.println("ci   = " + ci);
         // Q(ei)^-1
@@ -747,7 +905,6 @@ public class PolyUtilTest extends TestCase {
         r = PolyUtil.<ModInteger> interpolate(cfac, r, Q, fi, ai, ei);
         //System.out.println("r   = " + r);
 
-
         // next evaluation polynomial
         Qp = cfac.univariate(0);
         Qp = Qp.subtract(cfac.getONE().multiply(ei));
@@ -768,6 +925,8 @@ public class PolyUtilTest extends TestCase {
         //System.out.println("gi   = " + gi);
         hi = PolyUtil.<ModInteger> evaluateMain(fac, r, di);
         //System.out.println("hi   = " + hi);
+        assertTrue("gi == 1 ", gi.isONE());
+        assertTrue("hi == 1 ", hi.isONE());
 
         //            interpolate( a(ei), a(di) )            = a (mod 19)
         assertEquals("interpolate(a mod (x-ei),a mod (x-di)) = a (mod 19)", a, r);
@@ -776,7 +935,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test interpolate univariate deg > 0 polynomial.
-     * 
      */
     public void testInterpolateUnivariate() {
         ModInteger ai, ci, ei, fi;
@@ -854,7 +1012,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test interpolate multivariate deg > 0 polynomial.
-     * 
      */
     public void testInterpolateMultivariate() {
         ModInteger ci, ei, fi;
@@ -952,7 +1109,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test interpolate rational multivariate deg > 0 polynomial.
-     * 
      */
     public void testInterpolateRationalMultivariate() {
         BigRational ci, ei, fi;
@@ -1045,7 +1201,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test coefficient map function.
-     * 
      */
     public void testMap() {
         // integers
@@ -1078,7 +1233,7 @@ public class PolyUtilTest extends TestCase {
         //System.out.println("pi = " + pi);
 
         // random polynomial
-        GenPolynomial<BigRational> pr = pfr.random(kl, 2 * ll, el, q);
+        GenPolynomial<BigRational> pr = pfr.random(kl, 2 * ll, el, q).monic();
         //System.out.println("pr = " + pr);
 
         // random polynomial
@@ -1094,6 +1249,15 @@ public class PolyUtilTest extends TestCase {
         //System.out.println("qi = " + qi);
         assertEquals("pi == qi ", pi, qi);
 
+        // test rational to integer and back
+        qi = PolyUtil.integerFromRationalCoefficients(pfi, pr);
+        qr = PolyUtil.<BigInteger, BigRational> map(pfr, qi, new FromInteger<BigRational>(fr));
+        qr = qr.monic();
+        //System.out.println("pr = " + pr);
+        //System.out.println("qr = " + qr);
+        //System.out.println("qi = " + qi);
+        assertEquals("pr == qr ", pr, qr);
+
         // test symmetric modular integer to integer and back
         GenPolynomial<ModInteger> qm;
         qi = PolyUtil.<ModInteger, BigInteger> map(pfi, pm, new ModSymToInt<ModInteger>());
@@ -1122,7 +1286,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test substitution.
-     * 
      */
     public void testSubstitution() {
         dfac = new GenPolynomialRing<BigInteger>(new BigInteger(1), 1, to);
@@ -1148,7 +1311,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test algebraic substitution.
-     * 
      */
     public void testAlgebraicSubstitution() {
 
@@ -1159,18 +1321,18 @@ public class PolyUtilTest extends TestCase {
         GenPolynomial<BigRational> agen = pfac.univariate(0, 2);
         agen = agen.sum(pfac.getONE()); // x^2 + 1
         AlgebraicNumberRing<BigRational> afac = new AlgebraicNumberRing<BigRational>(agen, true);
-        GenPolynomialRing<AlgebraicNumber<BigRational>> apfac = new GenPolynomialRing<AlgebraicNumber<BigRational>>(
-                afac, 1, to, vars); // univariate
+        GenPolynomialRing<AlgebraicNumber<BigRational>> apfac 
+           = new GenPolynomialRing<AlgebraicNumber<BigRational>>(afac, 1, to, vars); // univariate
 
         //System.out.println("agen  = " + agen);
         //System.out.println("afac  = " + afac);
         //System.out.println("apfac = " + apfac);
 
         // subs = x - 7
-        GenPolynomial<AlgebraicNumber<BigRational>> s = apfac.univariate(0).subtract(
-                apfac.fromInteger(7).multiply(afac.getGenerator()));
-        GenPolynomial<AlgebraicNumber<BigRational>> s1 = apfac.univariate(0).sum(
-                apfac.fromInteger(7).multiply(afac.getGenerator()));
+        GenPolynomial<AlgebraicNumber<BigRational>> s 
+           = apfac.univariate(0).subtract(apfac.fromInteger(7).multiply(afac.getGenerator()));
+        GenPolynomial<AlgebraicNumber<BigRational>> s1 
+           = apfac.univariate(0).sum(apfac.fromInteger(7).multiply(afac.getGenerator()));
         //System.out.println("s = " + s);
         //System.out.println("s1 = " + s1);
 
@@ -1190,14 +1352,13 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test switch variables.
-     * 
      */
     public void testSwitchVariables() {
 
         BigRational cfac = new BigRational(1);
         GenPolynomialRing<BigRational> pfac = new GenPolynomialRing<BigRational>(cfac, rl, to);
-        GenPolynomialRing<GenPolynomial<BigRational>> rfac = new GenPolynomialRing<GenPolynomial<BigRational>>(
-                pfac, rl, to);
+        GenPolynomialRing<GenPolynomial<BigRational>> rfac 
+           = new GenPolynomialRing<GenPolynomial<BigRational>>(pfac, rl, to);
 
         //System.out.println("pfac  = " + pfac);
         //System.out.println("rfac  = " + rfac);
@@ -1219,21 +1380,20 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test algebraic conversions.
-     * 
      */
     public void testAlgebraicConversions() {
 
         BigRational cfac = new BigRational(1);
         String[] alpha = new String[] { "alpha" };
-        String[] vars = new String[] { "z" };
+        //String[] vars = new String[] { "z" };
         GenPolynomialRing<BigRational> pfac = new GenPolynomialRing<BigRational>(cfac, 1, to, alpha);
         GenPolynomial<BigRational> agen = pfac.univariate(0, 2);
         agen = agen.sum(pfac.getONE()); // x^2 + 1
         AlgebraicNumberRing<BigRational> afac = new AlgebraicNumberRing<BigRational>(agen, true);
-        GenPolynomialRing<AlgebraicNumber<BigRational>> apfac = new GenPolynomialRing<AlgebraicNumber<BigRational>>(
-                afac, rl, to);
-        GenPolynomialRing<GenPolynomial<BigRational>> rfac = new GenPolynomialRing<GenPolynomial<BigRational>>(
-                pfac, rl, to);
+        GenPolynomialRing<AlgebraicNumber<BigRational>> apfac 
+           = new GenPolynomialRing<AlgebraicNumber<BigRational>>(afac, rl, to);
+        GenPolynomialRing<GenPolynomial<BigRational>> rfac 
+           = new GenPolynomialRing<GenPolynomial<BigRational>>(pfac, rl, to);
 
         //System.out.println("agen  = " + agen);
         //System.out.println("afac  = " + afac);
@@ -1258,7 +1418,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test Taylor series.
-     * 
      */
     public void testTaylorSeries() {
         GenPolynomial<BigRational> a;
@@ -1295,7 +1454,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test Complex real and imaginary part.
-     * 
      */
     public void testComplexParts() {
         BigRational rf = new BigRational(1);
@@ -1342,7 +1500,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test product represenation conversion, rational numbers.
-     * 
      */
     public void testProductConversionRN() {
         GenPolynomialRing<BigRational> ufac;
@@ -1374,7 +1531,6 @@ public class PolyUtilTest extends TestCase {
 
     /**
      * Test polynomal over product represenation conversion, algebraic numbers.
-     * 
      */
     public void testPolyProductConversionAN() {
         GenPolynomialRing<BigRational> ufac;
@@ -1417,4 +1573,110 @@ public class PolyUtilTest extends TestCase {
         assertTrue("!isONE( cp )", !cp.isONE());
     }
 
+
+    /**
+     * Test remove unused upper varaibles.
+     */
+    public void testRemoveUnusedUpper() {
+        //System.out.println("dfac = " + dfac);
+        a = dfac.univariate(3, 2);
+        b = a.subtract(dfac.univariate(1, 1));
+        //System.out.println("a = " + a);
+        //System.out.println("b = " + b);
+
+        c = PolyUtil.<BigInteger> removeUnusedUpperVariables(b);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 4: " + c.ring.nvar, c.ring.nvar == 4);
+
+        a = dfac.univariate(3, 2);
+        //System.out.println("a = " + a);
+
+        c = PolyUtil.<BigInteger> removeUnusedUpperVariables(a);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 2: " + c.ring.nvar, c.ring.nvar == 2);
+
+        a = dfac.univariate(1, 2);
+        //System.out.println("a = " + a);
+
+        c = PolyUtil.<BigInteger> removeUnusedUpperVariables(a);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 4: " + c.ring.nvar, c.ring.nvar == 4);
+    }
+
+
+    /**
+     * Test remove unused lower varaibles.
+     */
+    public void testRemoveUnusedLower() {
+        //System.out.println("dfac = " + dfac);
+        a = dfac.univariate(3, 2);
+        b = a.subtract(dfac.univariate(1, 1));
+        //System.out.println("a = " + a);
+        //System.out.println("b = " + b);
+
+        c = PolyUtil.<BigInteger> removeUnusedLowerVariables(b);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 4: " + c.ring.nvar, c.ring.nvar == 4);
+
+        a = dfac.univariate(3, 2);
+        //System.out.println("a = " + a);
+
+        c = PolyUtil.<BigInteger> removeUnusedLowerVariables(a);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 4: " + c.ring.nvar, c.ring.nvar == 4);
+
+        a = dfac.univariate(1, 2);
+        //System.out.println("a = " + a);
+
+        c = PolyUtil.<BigInteger> removeUnusedLowerVariables(a);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 2: " + c.ring.nvar, c.ring.nvar == 2);
+    }
+
+
+    /**
+     * Test remove unused middle varaibles.
+     */
+    public void testRemoveUnusedMiddle() {
+        //System.out.println("dfac = " + dfac);
+        a = dfac.univariate(4, 2);
+        b = a.subtract(dfac.univariate(0, 1));
+        //System.out.println("a = " + a);
+        //System.out.println("b = " + b);
+
+        c = PolyUtil.<BigInteger> removeUnusedLowerVariables(b);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 5: " + c.ring.nvar, c.ring.nvar == 5);
+        c = PolyUtil.<BigInteger> removeUnusedUpperVariables(c);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 5: " + c.ring.nvar, c.ring.nvar == 5);
+
+        c = PolyUtil.<BigInteger> removeUnusedMiddleVariables(c);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 2: " + c.ring.nvar, c.ring.nvar == 2);
+
+        a = dfac.univariate(3, 2);
+        b = a.subtract(dfac.univariate(1, 1));
+        //System.out.println("a = " + a);
+        //System.out.println("b = " + b);
+
+        try {
+             c = PolyUtil.<BigInteger> removeUnusedMiddleVariables(b);
+             fail("c = " + c + ", fac = " + c.ring);
+        } catch (RuntimeException e) {
+	    // success
+        }
+
+        c = PolyUtil.<BigInteger> removeUnusedLowerVariables(b);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 4: " + c.ring.nvar, c.ring.nvar == 4);
+        c = PolyUtil.<BigInteger> removeUnusedUpperVariables(c);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 3: " + c.ring.nvar, c.ring.nvar == 3);
+
+        c = PolyUtil.<BigInteger> removeUnusedMiddleVariables(c);
+        //System.out.println("c = " + c + ", fac = " + c.ring);
+        assertTrue("#var == 2: " + c.ring.nvar, c.ring.nvar == 2);
+    }
+
 }
