@@ -1,5 +1,5 @@
 /*
- * $Id: GCDModularTest.java 3789 2011-10-01 18:54:43Z kredel $
+ * $Id: GCDModularTest.java 3843 2011-12-29 11:56:02Z kredel $
  */
 
 package edu.jas.ufd;
@@ -13,6 +13,8 @@ import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
 
+import org.apache.log4j.BasicConfigurator;
+
 import edu.jas.arith.BigInteger;
 import edu.jas.arith.ModInteger;
 import edu.jas.arith.ModIntegerRing;
@@ -35,6 +37,7 @@ public class GCDModularTest extends TestCase {
      * main.
      */
     public static void main(String[] args) {
+        BasicConfigurator.configure();
         junit.textui.TestRunner.run(suite());
     }
 
@@ -56,8 +59,6 @@ public class GCDModularTest extends TestCase {
     }
 
 
-    //private final static int bitlen = 100;
-
     GreatestCommonDivisorAbstract<ModInteger> ufd;
 
 
@@ -157,7 +158,7 @@ public class GCDModularTest extends TestCase {
         ai = bi = ci = di = ei = null;
         ar = br = cr = dr = er = null;
         //mi = new ModIntegerRing(primes.get(0), true);
-        mi = new ModIntegerRing(5L, true);
+        mi = new ModIntegerRing(19L, true);
         ufd = new GreatestCommonDivisorPrimitive<ModInteger>();
         dfac = new GenPolynomialRing<ModInteger>(mi, rl, to);
         cfac = new GenPolynomialRing<ModInteger>(mi, rl - 1, to);
@@ -179,12 +180,10 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test modular algorithm gcd with modular evaluation recursive algorithm.
-     * 
      */
     public void testModularEvaluationGcd() {
 
         GreatestCommonDivisorAbstract<BigInteger> ufd_m = new GreatestCommonDivisorModular(/*false*/);
-
         GreatestCommonDivisorAbstract<BigInteger> ufd = new GreatestCommonDivisorPrimitive<BigInteger>();
 
         GenPolynomial<BigInteger> a;
@@ -239,12 +238,10 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test modular algorithm gcd with simple PRS recursive algorithm.
-     * 
      */
     public void testModularSimpleGcd() {
 
         GreatestCommonDivisorAbstract<BigInteger> ufd_m = new GreatestCommonDivisorModular(true);
-
         GreatestCommonDivisorAbstract<BigInteger> ufd = new GreatestCommonDivisorPrimitive<BigInteger>();
 
         GenPolynomial<BigInteger> a;
@@ -299,7 +296,6 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test recursive content and primitive part, modular coefficients.
-     * 
      */
     public void testRecursiveContentPPmodular() {
 
@@ -351,7 +347,6 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test base gcd modular coefficients.
-     * 
      */
     public void testGCDbaseModular() {
 
@@ -401,7 +396,6 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test recursive gcd modular coefficients.
-     * 
      */
     public void testRecursiveGCDModular() {
 
@@ -458,7 +452,6 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test arbitrary recursive gcd modular coefficients.
-     * 
      */
     public void testArbitraryRecursiveGCDModular() {
 
@@ -515,7 +508,6 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test gcd modular coefficients.
-     * 
      */
     public void testGcdModular() {
 
@@ -568,7 +560,6 @@ public class GCDModularTest extends TestCase {
 
     /**
      * Test co-prime factors.
-     * 
      */
     public void testCoPrime() {
 
@@ -614,4 +605,165 @@ public class GCDModularTest extends TestCase {
         assertTrue("is co-prime of ", ufd.isCoPrime(P, F));
     }
 
+
+    /**
+     * Test base resultant modular coefficients.
+     */
+    public void testResultantBaseModular() {
+
+        dfac = new GenPolynomialRing<ModInteger>(mi, 1, to);
+
+        GreatestCommonDivisorSimple<ModInteger> ufds = new GreatestCommonDivisorSimple<ModInteger>();
+        GreatestCommonDivisorSubres<ModInteger> sres = new GreatestCommonDivisorSubres<ModInteger>();
+
+        for (int i = 0; i < 3; i++) {
+            a = dfac.random(kl, ll, el + 3 + i, q).monic();
+            b = dfac.random(kl, ll, el + 3 + i, q).monic();
+            c = dfac.random(kl, ll, el + 3 + i, q).monic();
+            //System.out.println("a = " + a);
+            //System.out.println("b = " + b);
+            //System.out.println("c = " + c);
+
+            if (a.isZERO() || b.isZERO() || c.isZERO()) {
+                // skip for this turn
+                continue;
+            }
+            if (c.isConstant()) {
+                c = dfac.univariate(0,1);
+            }
+            assertTrue("length( c" + i + " ) <> 0", c.length() > 0);
+
+            d = ufds.baseResultant(a, b);
+            //System.out.println("d = " + d);
+            e = sres.baseResultant(a, b);
+            //System.out.println("e = " + e);
+            assertEquals("d == e: " + d.subtract(e), d.signum(), e.signum());
+            //assertEquals("d == e: " + d.subtract(e), d, e);
+
+            ac = a.multiply(c);
+            bc = b.multiply(c);
+            //System.out.println("ac = " + ac);
+            //System.out.println("bc = " + bc);
+
+            d = ufds.baseResultant(ac, bc);
+            //System.out.println("d = " + d);
+            assertTrue("d == 0: " + d, d.isZERO());
+
+            e = sres.baseResultant(ac, bc);
+            //System.out.println("e = " + e);
+            assertTrue("e == 0: " + e, e.isZERO());
+
+            //assertEquals("d == e: " + d.subtract(e), d, e);
+        }
+    }
+
+
+    /**
+     * Test recursive resultant modular coefficients.
+     */
+    public void testRecursiveResultantModular() {
+
+        cfac = new GenPolynomialRing<ModInteger>(mi, 2 - 0, to);
+        rfac = new GenPolynomialRing<GenPolynomial<ModInteger>>(cfac, 1, to);
+        //System.out.println("rfac = " + rfac);
+ 
+        GreatestCommonDivisorSimple<ModInteger> ufds = new GreatestCommonDivisorSimple<ModInteger>();
+        GreatestCommonDivisorSubres<ModInteger> sres = new GreatestCommonDivisorSubres<ModInteger>();
+
+        for (int i = 0; i < 1; i++) {
+            ar = rfac.random(kl, 2, el + 2, q);
+            br = rfac.random(kl, 2, el + 3, q);
+            cr = rfac.random(kl, 2, el + 1, q);
+            ar = PolyUtil.<ModInteger> monic(ar);
+            br = PolyUtil.<ModInteger> monic(br);
+            cr = PolyUtil.<ModInteger> monic(cr);
+            //System.out.println("ar = " + ar);
+            //System.out.println("br = " + br);
+            //System.out.println("cr = " + cr);
+
+            if (ar.isZERO() || br.isZERO() || cr.isZERO()) {
+                // skip for this turn
+                continue;
+            }
+            if (cr.isConstant()) {
+                cr = rfac.univariate(0,1);
+            }
+            assertTrue("length( cr" + i + " ) <> 0", cr.length() > 0);
+
+            dr = ufds.recursiveUnivariateResultant(ar, br);
+            //System.out.println("dr = " + dr);
+            er = sres.recursiveUnivariateResultant(ar, br);
+            //System.out.println("er = " + er);
+            assertEquals("dr == er: " + dr.subtract(er), dr.signum(), er.signum());
+            //assertEquals("dr == er: " + dr.subtract(er), dr, er);
+
+            arc = ar.multiply(cr);
+            brc = br.multiply(cr);
+            //System.out.println("arc = " + arc);
+            //System.out.println("brc = " + brc);
+
+            dr = ufds.recursiveUnivariateResultant(arc, brc);
+            //System.out.println("dr = " + dr);
+            //assertTrue("dr == 0: " + dr, dr.isZERO());
+
+            er = sres.recursiveUnivariateResultant(arc, brc);
+            //System.out.println("er = " + er);
+            //assertTrue("er == 0: " + er, er.isZERO());
+
+            assertEquals("dr == er: " + dr.subtract(er), dr.signum(), er.signum());
+        }
+    }
+
+
+    /**
+     * Test resultant modular coefficients.
+     */
+    public void testResultantModular() {
+        dfac = new GenPolynomialRing<ModInteger>(mi, 4, to);
+        //System.out.println("dfac = " + dfac);
+
+        GreatestCommonDivisorSimple<ModInteger> ufds = new GreatestCommonDivisorSimple<ModInteger>();
+        GreatestCommonDivisorSubres<ModInteger> sres = new GreatestCommonDivisorSubres<ModInteger>();
+
+        for (int i = 0; i < 1; i++) {
+            a = dfac.random(kl, ll, el + i, q).monic();
+            b = dfac.random(kl, ll, el + i, q).monic();
+            c = dfac.random(kl, ll, el + i, q).monic();
+            //System.out.println("a = " + a);
+            //System.out.println("b = " + b);
+            //System.out.println("c = " + c);
+
+            if (a.isZERO() || b.isZERO() || c.isZERO()) {
+                // skip for this turn
+                continue;
+            }
+            if (c.isConstant()) {
+                c = dfac.univariate(0,1);
+            }
+            assertTrue("length( c" + i + " ) <> 0", c.length() > 0);
+
+            d = ufds.resultant(a, b);
+            //System.out.println("d = " + d);
+            e = sres.resultant(a, b);
+            //System.out.println("e = " + e);
+            assertEquals("d == e: " + d.subtract(e), d.signum(), e.signum());
+            //assertEquals("d == e: " + d.subtract(e), d, e);
+
+            ac = a.multiply(c);
+            bc = b.multiply(c);
+            //System.out.println("ac = " + ac);
+            //System.out.println("bc = " + bc);
+
+            d = ufds.resultant(ac, bc);
+            //System.out.println("d = " + d);
+            //assertTrue("d == 0: " + d, d.isZERO());
+
+            e = sres.resultant(ac, bc);
+            //System.out.println("e = " + e);
+            //assertTrue("e == 0: " + e, e.isZERO());
+
+            assertEquals("d == e: " + d.subtract(e), d.signum(), e.signum());
+        }
+    }
+
 }
